#!/usr/bin/env python
# Author: Carsten Sachse 21-Sep-2010
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
* Several classes to define features of programs
"""
from collections import namedtuple
from glob import glob
from multiprocessing import cpu_count
from spring.csinfrastr.cslogger import GetMetaData
from collections import OrderedDict
import os
import time

class FeaturesSupport(object):

    def add_file_extensions_in_comma_separated_string(self, feature_set, inp3):
        return ', {0}'.format(os.extsep).join(feature_set.properties[inp3].ext)

    def add_accepted_file_formats_to_hint(self, feature_set, inp3):
        return 'accepted file formats ({0}).'.\
        format(self.add_file_extensions_in_comma_separated_string(feature_set, inp3))


    def get_diagnostic_output_formats_for_matplotlib(self):
        return ['pdf', 'png', 'bmp', 'emf', 'eps', 'gif', 'jpeg', 'jpg', 'ps', 'raw', 'rgba', 'svg', 'svgz', 'tif',
        'tiff']


    def get_input_micrograph_formats(self):
        return ['tif', 'mrc', 'mrcs', 'spi', 'hdf', 'img', 'hed']
    

    def get_cpu_hint(self):
        return 'Number of processors to be used.'

class FeaturesBinning(object):

    def get_binning_hint(self, image):
        if image in ['micrograph']:
            hint_txt = 'Micrograph is '
        if image in ['segments']:
            hint_txt = 'Segments are '
            
        hint_txt += 'reduced in size by binning'
        
        return hint_txt
    

    def set_binning_option(self, feature_set, default=False, image='micrograph'):
        inp15 = 'Binning option'
        feature_set.parameters[inp15]=bool(default)
        feature_set.hints[inp15]=self.get_binning_hint(image) + '.'
        feature_set.level[inp15]='intermediate'
        
        return feature_set
        
        
    def set_binning_factor(self, feature_set, binfactor=6, image='micrograph'):
        inp16 = 'Binning factor'
        feature_set.parameters[inp16]=int(binfactor)
        feature_set.hints[inp16]=self.get_binning_hint(image) + ' factor.'
        feature_set.properties[inp16]=feature_set.Range(1,20,1)
        feature_set.relatives[inp16]='Binning option'
        feature_set.level[inp16]='intermediate'
        
        return feature_set
    
    
    def set_internal_binning(self, feature_set):
        inp9 = 'Internal binning factor'
        feature_set.parameters[inp9]=2
        feature_set.hints[inp9]='In order to speed up alignments images can be optionally binned.'
        feature_set.properties[inp9]=feature_set.Range(1,20,1)
        feature_set.level[inp9]='intermediate'
        
        return feature_set
    
    
class FeaturesFilter(object):
    def set_high_pass_filter_option(self, feature_set):
        inp15 = 'High-pass filter option'
        feature_set.parameters[inp15] = bool(False)
        feature_set.hints[inp15] = 'Option to high-pass filter images before alignment or band-pass filter in ' + \
        'combination with low-pass filter.'
        feature_set.level[inp15]='intermediate'
        
        return feature_set
    

    def set_low_pass_filter_option(self, feature_set):
        inp10 = 'Low-pass filter option'
        feature_set.parameters[inp10] = bool(True)
        feature_set.hints[inp10] = 'Option to low-pass filter images before alignment or band-pass filter in ' + \
        'combination with high-pass filter.'
        feature_set.level[inp10]='intermediate'

        return feature_set
    
    
    def set_high_and_low_pass_filter_cutoff(self, feature_set):
        inp11 = 'High and low-pass filter cutoffs in 1/Angstrom'
        feature_set.parameters[inp11] = tuple((0.001, 0.09))
        feature_set.hints[inp11] = 'Filter design for high- and low-pass filter with cutoffs in 1/Angstrom. ' + \
        'Maximum spatial frequency is 1/(2*pixelsize).'
        feature_set.properties[inp11] = feature_set.Range(0, 1, 0.001)
        feature_set.relatives[inp11] = ('Low-pass filter option', 'High-pass filter option')
        feature_set.level[inp11]='intermediate'

        return feature_set
    

    def set_custom_filter_option(self, feature_set):
        inp12 = 'Custom filter option'
        feature_set.parameters[inp12] = bool(False)
        feature_set.hints[inp12] = 'Option to custom filter images before alignment.'
        feature_set.level[inp12]='expert'

        return feature_set
    
    
    def set_custom_build_filter_file(self, feature_set):
        inp13 = 'Custom-built filter file'
        feature_set.parameters[inp13] = 'filter_function.dat'
        feature_set.properties[inp13] = feature_set.file_properties(1, ['txt', 'dat'], 'getFile')
        feature_set.hints[inp13] = 'Custom-built filter function with two columns (normalized spatial frequency ' + \
        '0 - 0.5, Fourier filter coefficients between 0 - 1): ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp13) 
        
        feature_set.relatives[inp13] = 'Custom filter option'
        feature_set.level[inp13]='expert'

        return feature_set
    
    
    def set_bfactor_on_images(self, feature_set):
        inp10 = 'B-Factor'
        feature_set.parameters[inp10] = int(0)
        feature_set.properties[inp10]=feature_set.Range(-50000, 50000, 10)
        feature_set.hints[inp10] = 'B-Factor in 1/Angstrom^2 to be applied to images. Zero no modulation of ' + \
        'frequencies. Negative B-factors enhance high-resolution frequencies. Positive B-factors dampen ' + \
        'high-resolution frequencies.'
        feature_set.level[inp10]='expert'

        return feature_set


    def set_layer_line_filter_option(self, feature_set):
        inp5 = 'Filter layer-lines option'
        feature_set.parameters[inp5]=bool(False)
        feature_set.hints[inp5]='Tick if reference is to be filtered iteratively by layer-line mask based on ' + \
        'symmetry parameters.'
        feature_set.level[inp5]='experimental'
        
        return feature_set
    
    
    def set_filter_options(self, feature_set):
        feature_set = self.set_high_pass_filter_option(feature_set)
        feature_set = self.set_low_pass_filter_option(feature_set)
        feature_set = self.set_high_and_low_pass_filter_cutoff(feature_set)
        feature_set = self.set_bfactor_on_images(feature_set)
        feature_set = self.set_custom_filter_option(feature_set)
        feature_set = self.set_custom_build_filter_file(feature_set)
        
        return feature_set
    

class FeaturesAlign(object):
    def set_alignment_pixel_step(self, feature_set):
        inp6 = 'Step size of alignment in pixels'
        feature_set.parameters[inp6] = int(1)
        feature_set.hints[inp6] = 'Alignment search is performed every ...th pixel, higher numbers speed up the ' + \
        'procedure at the cost of alignment accuracy.'
        feature_set.properties[inp6] = feature_set.Range(1, 50, 1)
        feature_set.level[inp6]='expert'
        
        return feature_set


    def set_restrain_in_plane_angular_search_option(self, feature_set):
        inp15 = 'Limit in-plane rotation'
        feature_set.parameters[inp15]=bool(True)
        feature_set.hints[inp15]='Restricts in-plane rotation alignment.'
        feature_set.level[inp15]='intermediate'
        
        return feature_set
    
        
    def set_delta_in_plane_angular_search(self, feature_set):
        inp7 = 'Delta in-plane rotation angle'
        feature_set.parameters[inp7] = 10.0
        feature_set.hints[inp7] = 'Rotational alignment is being performed within +/- delta angle from 0 and 180 ' + \
        'degrees (delta=0 means only 0/180 are checked, delta=180 means no rotational restraint).'
        feature_set.properties[inp7] = feature_set.Range(0.0, 180.0, 1)
        feature_set.relatives[inp7] = 'Limit in-plane rotation'
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    

    def set_x_y_translation(self, feature_set):
        inp8 = 'X and Y translation range in Angstrom'
        feature_set.parameters[inp8]=tuple((50, 50))
        feature_set.hints[inp8]='Translational range of alignment search perpendicular and along helix axis (X: ' + \
        'off-center helical axis, Y: (minimum y-range=helical rise/2).'
        feature_set.properties[inp8]=feature_set.Range(0, 1000, 1)
        feature_set.level[inp8]='intermediate'
        
        return feature_set
    
        
    def set_alignment_rotation_and_translation(self, feature_set):
        feature_set = self.set_restrain_in_plane_angular_search_option(feature_set)
        feature_set = self.set_delta_in_plane_angular_search(feature_set)
        feature_set = self.set_x_y_translation(feature_set)
        
        return feature_set


    def set_absolute_translation_limit(self, feature_set, relative=None):
        inp8 = 'Absolute X and Y translation limit in Angstrom'
        feature_set.parameters[inp8]=tuple((100, 100))
        feature_set.hints[inp8]='Absolute translation limit of alignment search in Angstrom perpendicular ' + \
        'and along helix axis from center of segment. This is useful because iterative alignments tend to move ' + \
        'the helix axis in y-direction.' 
        feature_set.properties[inp8]=feature_set.Range(1, 1000, 1)
        if relative is not None:
            feature_set.relatives[inp8] = (relative, relative)
        feature_set.level[inp8]='intermediate'
        
        return feature_set
    
    
    def set_center_and_rotation_option(self, feature_set):
        inp7 = 'Center option'
        feature_set.parameters[inp7] = bool(True)
        feature_set.hints[inp7] = 'Class averages are centered and rotationally aligned with respect to their helix ' +\
        'axis.'
        feature_set.level[inp7]='expert'

        return feature_set
    

class FeaturesHelix(object):
    def set_pixelsize(self, feature_set):
        inp7 = 'Pixel size in Angstrom'
        feature_set.parameters[inp7]=float(1.163)
        feature_set.hints[inp7]='Pixel size is an imaging parameter.'
        feature_set.properties[inp7]=feature_set.Range(0.001, 100, 0.001)
        feature_set.level[inp7]='beginner'
        
        return feature_set


    def set_helix_width(self, feature_set):
        inp5 = 'Estimated helix width in Angstrom'
        feature_set.parameters[inp5]=int(200)
        feature_set.hints[inp5]='Generous width measure of helix required for rectangular mask.'
        feature_set.properties[inp5]=feature_set.Range(0,1500,10)
        feature_set.level[inp5]='beginner'
        
        return feature_set
    

    def set_helix_width_and_height(self, feature_set):
        inp5 = 'Estimated helix width and height in Angstrom'
        feature_set.parameters[inp5]=((200, 600))
        feature_set.hints[inp5]='Generous width and height measure of helix required for rectangular mask.'
        feature_set.properties[inp5]=feature_set.Range(20,1500,10)
        feature_set.level[inp5]='beginner'
        
        return feature_set
     
     
    def set_exact_helix_width(self, feature_set):
        inp5 = 'Precise helix width in Angstrom'
        feature_set.parameters[inp5]=int(180)
        feature_set.hints[inp5]='Precise width of helix for layer line interpretation.'
        feature_set.properties[inp5]=feature_set.Range(20,1500,10)
        feature_set.level[inp5]='beginner'
        
        return feature_set
    

    def set_helix_inner_outer_diameter(self, feature_set):
        inp5 = 'Estimated helix inner and outer diameter in Angstrom'
        feature_set.parameters[inp5]=tuple((int(0), int(190)))
        feature_set.hints[inp5]='Generous inner and outer diameter of helix required for cylindrical mask in Angstrom.'
        feature_set.properties[inp5]=feature_set.Range(0,1500,10)
        feature_set.level[inp5]='beginner'
        
        return feature_set


    def set_power_cutoff(self, feature_set):
        inp6 = 'Power spectrum resolution cutoff in 1/Angstrom'
        feature_set.parameters[inp6]=float(0.15)
        feature_set.hints[inp6]='Images are binned to obtain a suitable power spectrum.'
        feature_set.properties[inp6]=feature_set.Range(0,1,0.01)
        feature_set.level[inp6]='expert'
        
        return feature_set


class FeaturesHelixSymmetry(FeaturesHelix):
    def set_rise_rotation_or_pitch_unitnumber_choice(self, feature_set):
        inp7 = 'Helical rise/rotation or pitch/number of units per turn choice'
        feature_set.parameters[inp7] = str('rise/rotation')
        feature_set.hints[inp7] = 'Choose whether helical \'rise/rotation\' or \'pitch/unit_number\' of units per ' + \
        'turn pairs are given for generating the helical lattice.'
        
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['rise/rotation', 'pitch/unit_number'],
        'QComboBox')
        
        feature_set.level[inp7]='beginner'

        return feature_set
    
    
    def set_helical_rise_search_range(self, feature_set):
        inp6 = 'Range of helical rise or pitch search in Angstrom'
        feature_set.parameters[inp6] = tuple((1.0, 10.0))
        feature_set.hints[inp6] = 'Helical rise or pitch (Angstrom) range (from ... to ...) to be reconstructed.'
        feature_set.properties[inp6] = feature_set.Range(0, 1000, 0.0001)
        feature_set.level[inp6]='beginner'
        
        return feature_set


    def set_helical_rotation_search_range(self, feature_set):
        inp7 = 'Range of helical rotation in degrees or number of units per turn search'
        feature_set.parameters[inp7] = tuple((1.0, 300.0))
        feature_set.hints[inp7] = 'Helical rotation (degrees) or \'number of units per turn\' range (from ... to ' + \
        '...) to be reconstructed.'
        feature_set.properties[inp7] = feature_set.Range(-360, 360, 0.0001)
        feature_set.level[inp7]='beginner'
        
        return feature_set


    def set_increment_helical_symmetry_search(self, feature_set):
        inp8 = 'Increments of helical symmetry steps in Angstrom or degrees'
        feature_set.parameters[inp8] = tuple((1.0, 0.1))
        feature_set.hints[inp8] = 'Helical rise (Angstrom) and rotation (degrees) increments to be reconstructed.'
        feature_set.properties[inp8] = feature_set.Range(0, 100, 0.001)
        feature_set.level[inp8]='beginner'
        
        return feature_set


    def defined_shared_helical_symmetry_value_range_and_level(self, feature_set, inp8):
        feature_set.properties[inp8] = feature_set.Range(-1000, 1000, 0.001)
        feature_set.level[inp8] = 'beginner'
        
        return feature_set


    def set_helical_symmetry(self, feature_set):
        inp8 = 'Helical symmetry in Angstrom or degrees'
        feature_set.parameters[inp8] = tuple((1.408, 22.03))
        feature_set.hints[inp8] = 'Helical rise/pitch (Angstrom) and rotation (degrees)/number of units per turn ' + \
        'to be imposed to 3D reconstruction.'
        feature_set = self.defined_shared_helical_symmetry_value_range_and_level(feature_set, inp8)
        
        return feature_set


    def set_rotational_symmetry(self, feature_set):
        inp7 = 'Rotational symmetry'
        feature_set.parameters[inp7] = int(1)
        feature_set.hints[inp7] = 'Additional x-fold rotational symmetry or x-number of helix start.'
        feature_set.properties[inp7] = feature_set.Range(1, 100, 1)
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    

    def set_polar_apolar_helix_choice(self, feature_set):
        inp7 = 'Helix polarity'
        feature_set.parameters[inp7] = str('polar')
        feature_set.hints[inp7] = 'Choose whether helix is \'polar\' or \'apolar\'. Polar helices have different ' + \
        'ends at the top and bottom. Only the predominant direction within a helix will be used for the ' + \
        'reconstruction. In apolar helices they are related by 180 degree rotation. Thus each segment can be ' + \
        'inserted twice in the 3D reconstruction in opposite directions.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['polar', 'apolar'], 'QComboBox')
        feature_set.level[inp7]='intermediate'

        return feature_set
    

class FeaturesHelix3dReconstruction(FeaturesHelixSymmetry):
    def set_volume_size_to_be_reconstructed(self, feature_set):
        inp5 = 'Percent of image reconstruction size'
        feature_set.parameters[inp5] = int(50)
        feature_set.hints[inp5] = 'Percent of image dimension to be used for helical reconstruction, i.e. number ' + \
        'of views inserted in reconstruction = (100 - percent)*imagesize in Angstrom / helical rise)'
        
        feature_set.properties[inp5] = feature_set.Range(10, 100, 1)
        feature_set.level[inp5]='expert'

        return feature_set


    def set_helical_symmetry_option(self, feature_set):
        inp5 = 'Symmetrize helix'
        feature_set.parameters[inp5]=bool(True)
        feature_set.hints[inp5]='Tick to impose helical symmetry. Symmetry-related views are incorporated into ' + \
        'final reconstruction.'
        feature_set.level[inp5]='beginner'
        
        return feature_set
    
    
    def set_helical_symmetry_reconstruction(self, feature_set, turn_on=False):
        feature_set = self.set_helix_inner_outer_diameter(feature_set)
        feature_set = self.set_pixelsize(feature_set)
        if turn_on:
            feature_set = self.set_helical_symmetry_option(feature_set)
        feature_set = self.set_rise_rotation_or_pitch_unitnumber_choice(feature_set)
        
        return feature_set
    
    
    def set_helical_symmetry_reconstruction_series(self, feature_set):
        feature_set = self.set_helical_symmetry_reconstruction(feature_set)
        feature_set = self.set_helical_rise_search_range(feature_set)
        feature_set = self.set_helical_rotation_search_range(feature_set)
        feature_set = self.set_increment_helical_symmetry_search(feature_set)
        
        return feature_set



    def set_out_of_plane_tilt_angle(self, feature_set):

        inp7 = 'Out-of-plane tilt angle'
        feature_set.parameters[inp7] = int(0)
        feature_set.hints[inp7] = 'Out-of-plane tilt angle in degrees (0=no out-of-plane tilt) of class average ' + \
        'view used for 3D reconstruction.'
        feature_set.properties[inp7] = feature_set.Range(-40, 40, 1)
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    
    
    def set_out_of_plane_tilt_angle_range(self, feature_set):

        inp7 = 'Out-of-plane tilt angle range'
        feature_set.parameters[inp7] = ((-12, 12))
        feature_set.hints[inp7] = 'Expected out-of-plane tilt angle in degrees (0=no out-of-plane tilt) of helices ' + \
        'used for 3D reconstruction.'
        feature_set.properties[inp7] = feature_set.Range(-40, 40, 1)
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    
    
    def set_angular_projection_count(self, feature_set):
        inp7 = 'Number of projections azimuthal/out-of-plane angle'
        feature_set.parameters[inp7] = ((90, 7))
        feature_set.hints[inp7] = 'Number of projections (e.g. 90 azimuthal projections per 360 degrees ' + \
        '- projection every four degrees, out-of-plane tilt 7 projections from -12 to 12 ' + \
        'degrees to be searched for 3D reconstruction. High- and maximum resolution reconstructions will be further ' +\
        'refined by 5 * number of specified azimuthal and out-of-plane angles.'
        feature_set.properties[inp7] = feature_set.Range(1, 1000, 1)
        feature_set.level[inp7]='intermediate'
        
        return feature_set
    

    def set_keep_intermediate_files_option(self, feature_set, hinttxt=None):
        inp15 = 'Keep intermediate files'
        feature_set.parameters[inp15]=bool(False)
        if hinttxt is None:
            hinttxt ='Keep intermediate projection and diagnostic images (power spectra ' + \
            'and reprojections) which are iteratively generated - EM image stacks are deleted otherwise.'
        feature_set.hints[inp15]=hinttxt
        
        feature_set.level[inp15]='intermediate'
        
        return feature_set
    

class FeaturesParticle(object):
    def set_particle_inner_and_outer_diameter(self, feature_set):
        inp5 = 'Estimated inner and outer particle diameter in Angstrom'
        feature_set.parameters[inp5]=((0, 160))
        feature_set.hints[inp5]='Generous inner and outer diameter measure of particle in Angstrom required for 2D ' + \
        'circular mask (inner diameter > 0 is very rare).'
        feature_set.properties[inp5]=feature_set.Range(0,1500,10)
        feature_set.level[inp5]='beginner'
        
        return feature_set
    
    
class FeaturesPower(object):
    def set_power_tile_size(self, feature_set, size=1800, power_of_2_hint=False):
        inp4 = 'Tile size power spectrum in Angstrom'
        feature_set.parameters[inp4] = int(size)
        feature_set.hints[inp4] = 'Tile size to be used for analysis.'
        if power_of_2_hint:
            feature_set.hints[inp4]+=' It will be internally converted to nearest power-of-two pixel dimensions.'
        feature_set.properties[inp4] = feature_set.Range(1, 10000, 1)
        feature_set.level[inp4]='expert'
        
        return feature_set


    def set_tile_overlap(self, feature_set, percent=50):
        inp5 = 'Tile overlap in percent'
        feature_set.parameters[inp5] = int(percent)
        feature_set.hints[inp5] = 'Overlap influences degree of averaging.'
        feature_set.properties[inp5] = feature_set.Range(0, 90, 1)
        feature_set.level[inp5]='expert'
        
        return feature_set


    def set_input_power_to_analyze(self, feature_set):
        inp1 = 'Analyze power spectrum'
        feature_set.parameters[inp1]=bool(True)
        feature_set.hints[inp1]='Option to analyze experimental power spectrum.'
        feature_set.level[inp1]='beginner'
        
        return feature_set
    
        
    def set_input_power_spectrum(self, feature_set, related=None, level='beginner'):
        inp1 = 'Power spectrum input image'
        feature_set.parameters[inp1]='power.hdf'
        feature_set.properties[inp1]=feature_set.file_properties(1,['hdf'],'getFile')
        feature_set.hints[inp1]='Input power spectrum image: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        feature_set.level[inp1]=level
        if related is not None:
            feature_set.relatives[inp1]=related
        
        return feature_set
    
    
    def set_output_power_spectrum(self, feature_set):
        inp1 = 'Power spectrum output image'
        feature_set.parameters[inp1]='power.hdf'
        feature_set.properties[inp1]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp1]='Output power spectrum image: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        
        return feature_set
    
    
    def set_output_enhanced_power_spectrum(self, feature_set):
        inp1 = 'Enhanced power spectrum output image'
        feature_set.parameters[inp1]='power_enhanced.hdf'
        feature_set.properties[inp1]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp1]='Output enhanced power spectrum image (compensated for decay of amplitudes): ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.relatives[inp1]='Enhanced power spectrum option'
        feature_set.level[inp1]='expert'
        
        return feature_set
    
    
    def set_enhance_power_option(self, feature_set):
        inp10 = 'Enhanced power spectrum option'
        feature_set.parameters[inp10] = bool(True)
        feature_set.hints[inp10] = 'Option to enhance power spectrum to compensate for decay of amplitudes.'
        feature_set.level[inp10]='expert'

        return feature_set
        

class FeaturesMicrograph(object):

    def set_inp_micrograph(self, feature_set):
        inp1 = 'Micrograph'
        feature_set.parameters[inp1]='cs_scan034.tif'
        feature_set.properties[inp1]=feature_set.file_properties(1, 
        FeaturesSupport().get_input_micrograph_formats(),'getFile')
        
        feature_set.hints[inp1]='Input micrograph: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        
        return feature_set
    

    def set_inp_multiple_micrographs(self, feature_set):
        inp1 = 'Micrographs'
        feature_set.parameters[inp1]='cs_scan034.tif'
        
        feature_set.properties[inp1]=feature_set.file_properties(10, FeaturesSupport().get_input_micrograph_formats(), 
        'getFiles')
        
        feature_set.hints[inp1]='Input micrographs: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'

        return feature_set
    
    
class FeaturesInteractive(object):
    def set_interactive_vs_batch_mode(self, feature_set):
        inp3 = 'Batch mode'
        feature_set.parameters[inp3]=bool(False)
        feature_set.hints[inp3]='Batch mode for plot. Otherwise interactive.'
        feature_set.level[inp3]='beginner'
        
        return feature_set
    
    
class FeaturesMpi(object):
    def set_mpi(self, feature_set):
        inp15 = 'MPI option'
        feature_set.parameters[inp15]=bool(True)
        feature_set.hints[inp15]='OpenMPI installed (mpirun).'
        feature_set.level[inp15]='intermediate'
        
        return feature_set


    def set_ncpus(self, feature_set):
        inp9 = 'Number of CPUs'
        feature_set.parameters[inp9]=cpu_count()
        feature_set.hints[inp9]=FeaturesSupport().get_cpu_hint()
        feature_set.properties[inp9]=feature_set.Range(1,1000,1)
        feature_set.relatives[inp9]='MPI option'
        feature_set.level[inp9]='intermediate'
        
        return feature_set
        
        
    def set_ncpus_scan(self, feature_set):
        inp9 = 'Number of CPUs'
        feature_set.parameters[inp9]=2
        feature_set.hints[inp9]=FeaturesSupport().get_cpu_hint() + ' Maximum number corresponds directly to number ' + \
        'of input scans, i.e. no gain in performance if single input micrograph chosen.'
        feature_set.properties[inp9]=feature_set.Range(1,300,1)
        feature_set.relatives[inp9]='MPI option'
        feature_set.level[inp9]='intermediate'
        
        return feature_set
        

class FeaturesSelectionAssist(object):
    def define_option(self, option_name, feature_set):
        inp7 = '{0} select option'.format(option_name.title())
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any particular {0}.'.format(option_name)
        feature_set.level[inp7]='expert'
        
        return feature_set
        

    def define_include_and_exclude(self, option_name, feature_set):
        inp7 = 'Include or exclude {0}'.format(option_name)
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' specified {0}.'.format(option_name)
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = '{0} select option'.format(option_name.title())
        feature_set.level[inp7]='expert'
        
        return feature_set
    

    def define_stringlist(self, option_name_plural, option_name_singular, feature_set, start=1):
        inp8 = '{0} list'.format(option_name_plural.title())
        feature_set.parameters[inp8] = str('1-9, 11, 13')
        feature_set.hints[inp8] = 'List of comma-separated {0} ids, e.g. \'1-10, 12, 14\' (1st {0} is {1}).'.\
        format(option_name_singular, start)
        feature_set.properties[inp8] = feature_set.file_properties(1, ['*'], None)
        feature_set.relatives[inp8] = '{0} select option'.format(option_name_plural.title())
        feature_set.level[inp8]='expert'
        
        return feature_set
    

class FeaturesSelection(FeaturesSelectionAssist):

    def set_in_or_exclude_segments_option(self, feature_set):
        self.define_option('segments', feature_set)
        
        return feature_set
    

    def set_in_or_exclude_segments(self, feature_set):
        feature_set = self.define_include_and_exclude('segments', feature_set)
        
        return feature_set


    def set_segments_from_segment_file_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Segment file'
        feature_set.parameters[inp8] = 'stackid_file.dat'
        feature_set.hints[inp8] = 'File with single column of stack_ids.'
        feature_set.properties[inp8]=feature_set.file_properties(2,['dat', 'txt'],'getFile')
        feature_set.relatives[inp8] = 'Segments select option'
        feature_set.level[inp8]='expert'
         
        return feature_set
    
        
        return feature_set


    def set_in_or_exclude_classes_option(self, feature_set):
        self.define_option('classes', feature_set)
        
        return feature_set
    

    def set_in_or_exclude_classes(self, feature_set):
        feature_set = self.define_include_and_exclude('classes', feature_set)
        
        return feature_set


    def set_classes_to_be_in_or_excluded(self, feature_set):
        feature_set = self.define_stringlist('classes', 'class', feature_set, start=0)
        
        return feature_set


    def set_class_type(self, feature_set):
        inp7 = 'Class type'
        feature_set.parameters[inp7] = str('class_id')
        feature_set.hints[inp7] = 'Choose class type either \'class_id\' based on segmentclass assignments or ' + \
        '\'class_model_id\' based on segmultirefine3d/segclassmodel assigments.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['class_id', 'class_model_id'], 'QComboBox')
        feature_set.relatives[inp7] = 'Classes select option'
        feature_set.level[inp7]='expert'
        
        return feature_set

    
    def set_persistence_class_selection_option(self, feature_set):
        inp7 = 'Persistence class option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select segments from classes based on class assignments of '+ \
        'neighboring segments.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    

    def set_persistence_class_length(self, feature_set):
        inp8 = 'Persistence class length in Angstrom'
        feature_set.parameters[inp8] = 700
        feature_set.hints[inp8] = 'Length of helix window that will be used to average the class memberships of segments.'
        feature_set.properties[inp8] = feature_set.Range(1,5000,10)
        feature_set.relatives[inp8] = 'Persistence class option'
        feature_set.level[inp8]='expert'
         
        return feature_set
    
    
    def set_persistence_class_occupancy(self, feature_set):
        inp8 = 'Class occupancy threshold'
        feature_set.parameters[inp8] = 0.5
        feature_set.hints[inp8] = 'Class occupancy threshold of segments (between 0 and 1) that belong to specified subset ' + \
        'of classes within persistence length. Segments that have a lower occupancy will be excluded.'
        feature_set.properties[inp8] = feature_set.Range(0,1,0.1)
        feature_set.relatives[inp8] = 'Persistence class option'
        feature_set.level[inp8]='expert'
         
        return feature_set
    
    
    def set_in_or_exclude_helices_option(self, feature_set):
        feature_set = self.define_option('helices', feature_set)
        
        return feature_set
    

    def set_in_or_exclude_helices(self, feature_set):
        feature_set = self.define_include_and_exclude('helices', feature_set)
        
        return feature_set

    
    def set_helices_to_be_in_or_excluded(self, feature_set):
        feature_set = self.define_stringlist('helices', 'helix', feature_set)
        
        return feature_set
    
    
    def set_in_or_exclude_micrographs_option(self, feature_set):
        feature_set = self.define_option('micrographs', feature_set)
        
        return feature_set
    

    def set_in_or_exclude_micrographs(self, feature_set):
        feature_set = self.define_include_and_exclude('micrographs', feature_set)
        
        return feature_set
    
    
    def set_micrographs_to_be_in_or_excluded(self, feature_set):
        feature_set = self.define_stringlist('micrographs', 'micrograph', feature_set)
        
        return feature_set
    
    
    def set_in_or_exclude_curvature_option(self, feature_set):
        inp7 = 'Straightness select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any helices based on straightness.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    

    def set_in_or_exclude_curvature(self, feature_set):
        inp7 = 'Include or exclude straight helices'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' helices of specified persistence length.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Straightness select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def get_percent_distribution_hint(self):
        txt='i.e. upper 10 percent of distribution is expressed as 90 - 100 percent range, ' + \
        'lower 20 percent is expressed as 0 - 20 percent etc. '
        return txt 


    def get_persistence_hint_txt(self):
        txt = 'Persistence length is calculated as: ' + \
        'p = -ln(2 * (end_to_end_distance / contour_length) ** 2 - 1) / contour_length)), i.e. short ' + \
        'persistence lengths of 1 nm correspond to very flexible whereas 1 m corresponds ' + \
        'to extremely straight helices. Examples are TMV: '+ \
        '2.9 mm (2.9e-3 m), amyloid beta filaments: 300 microm (3e-4 m) and DNA: 100 nm (1e-7 m). ' + \
        'Due to the alignment error of the segments this value may not be absolutely comparable to determined ' + \
        'persistence lengths by other methods but still be valid as a relative measure of straightness.'

        return txt 


    def set_curvature_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Persistence length range'
        feature_set.parameters[inp8]=((80, 100))
        feature_set.hints[inp8] = 'Range of persistence length in percent, ' + self.get_percent_distribution_hint() + \
        '90 - 100 % corresponds to most straight helices. ' + \
        'Values from database are stored in m, e.g. \'0-0.0001\' ' + \
        self.get_persistence_hint_txt()
        feature_set.properties[inp8]=feature_set.Range(0,100,1)
        feature_set.relatives[inp8]=(('Straightness select option', 'Straightness select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_persistence_length_cutoff(self, feature_set):
        inp9 = 'Persistence length cutoff in micrometer'
        feature_set.parameters[inp9] = float(300.0)
        feature_set.hints[inp9] = 'In case you specified a fit with an order higher than 1, the persistence length ' + \
        'can be used as a selection criterion. ' + self.get_persistence_hint_txt()
        feature_set.properties[inp9] = feature_set.Range(0, 1e+6, 100)
        feature_set.relatives[inp9] = 'Straightness select option'
        feature_set.level[inp9]='expert'
         
        return feature_set


    def set_in_or_exclude_defocus_option(self, feature_set):
        inp7 = 'Defocus select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on defocus.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_defocus(self, feature_set):
        inp7 = 'Include or exclude defocus range'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified defocus.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Defocus select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_defocus_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Defocus range'
        feature_set.parameters[inp8] = ((10000, 40000))
        feature_set.hints[inp8] = 'Range of defocus in Angstrom, e.g. \'10000-40000\''
        feature_set.properties[inp8] = feature_set.Range(0, 100000, 100)
        feature_set.relatives[inp8] = (('Defocus select option', 'Defocus select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_astigmatism_option(self, feature_set):
        inp7 = 'Astigmatism select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on astigmatism.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_astigmatism(self, feature_set):
        inp7 = 'Include or exclude astigmatic segments'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified astigmatism ' + \
        'amplitude in Angstrom.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Astigmatism select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_astigmatism_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Astigmatism range'
        feature_set.parameters[inp8] = ((0, 4000))
        feature_set.hints[inp8] = 'Range of astigmatism amplitude (difference between defocus one and two) in ' + \
        ' Angstrom, e.g. \'0-4000\'.'
        feature_set.properties[inp8] = feature_set.Range(0, 100000, 100)
        feature_set.relatives[inp8] = (('Astigmatism select option', 'Astigmatism select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_prj_ccc_option(self, feature_set):
        inp7 = 'Projection correlation select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on matched cross-correlation ' + \
        'coefficient.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_ccc_prj_match(self, feature_set):
        inp7 = 'Include or exclude segments based on projection correlation'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified cross ' + \
        'correlation coefficient with projection.' 
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Projection correlation select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_ccc_prj_match_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Correlation projection range'
        feature_set.parameters[inp8] = ((60, 100))
        feature_set.hints[inp8] = 'Range of cross-correlation peak between matched projection and segment in percent, ' + \
        self.get_percent_distribution_hint() + 'Values are stored as peak values depending on the pixel dimension, ' + \
        'e.g. 1220.'
        feature_set.properties[inp8] = feature_set.Range(0, 100, 1)
        feature_set.relatives[inp8] = (('Projection correlation select option', 'Projection correlation select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_layer_ccc_option(self, feature_set):
        inp7 = 'Layer line correlation select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on layer-line cross-correlation ' + \
        'coefficient.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_ccc_layer(self, feature_set):
        inp7 = 'Include or exclude segments based on layer-line correlation'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified cross ' + \
        'correlation coefficient with layer lines.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Layer line correlation select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_ccc_layer_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Correlation layer line range'
        feature_set.parameters[inp8] = ((60, 100))
        feature_set.hints[inp8] = 'Range of cross-correlation between layer lines of power spectrum average and ' + \
        'segment in percent. ' + self.get_percent_distribution_hint() + 'Values in database are stored as ' + \
        'cross correlation coefficient, e.g. \'0.5 - 1.0\'.'
        feature_set.properties[inp8] = feature_set.Range(0, 100, 1)
        feature_set.relatives[inp8] = (('Layer line correlation select option', 'Layer line correlation select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_out_of_plane_option(self, feature_set):
        inp7 = 'Out-of-plane tilt select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on out-of-plane tilt angle.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_out_of_plane_tilt(self, feature_set):
        inp7 = 'Include or exclude out-of-plane tilted segments'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified out-of-plane ' + \
        'tilt angle.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Out-of-plane tilt select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_out_of_plane_tilt_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Out-of-plane tilt range'
        feature_set.parameters[inp8] = ((-5, 5))
        feature_set.hints[inp8] = 'Range of out-of-plane tilt in degrees, e.g. \'-10 - 10 degrees\'.'
        feature_set.properties[inp8] = feature_set.Range(-50, 50, 0.5)
        feature_set.relatives[inp8] = (('Out-of-plane tilt select option', 'Out-of-plane tilt select option'))
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_helix_shift_x_option(self, feature_set):
        inp7 = 'Shift normal to helix select option'
        feature_set.parameters[inp7] = bool(False)
        feature_set.hints[inp7] = 'Choose whether to select any segments based on forward shift difference normal ' + \
        'to helix. This parameter tends to correlate with resolution.'
        feature_set.level[inp7]='expert'
        
        return feature_set
    
    
    def set_in_or_exclude_helix_shift_x(self, feature_set):
        inp7 = 'Include or exclude segments with shift normal to helix'
        feature_set.parameters[inp7] = str('include')
        feature_set.hints[inp7] = 'Choose whether to \'include\' or \'exclude\' segments of specified foward ' + \
        'shift difference normal to helix. \'include\' and 5 Angstrom  will select for segments with a ' + \
        'smaller forward difference. \'exclude\' and 5 Angstrom will select for segments with a larger forward ' + \
        'difference than 5 Angstrom.'
        feature_set.properties[inp7] = feature_set.choice_properties(2, ['include', 'exclude'], 'QComboBox')
        feature_set.relatives[inp7] = 'Shift normal to helix select option'
        feature_set.level[inp7]='expert'
        
        return feature_set


    def set_helix_shift_x_to_be_in_or_excluded(self, feature_set):
        inp8 = 'Shift normal to helix in Angstrom'
        feature_set.parameters[inp8] = float(5.0)
        feature_set.hints[inp8] = 'Shift normal to helix in Angstrom, e.g. \'5\'.'
        feature_set.properties[inp8] = feature_set.Range(0, 50, 0.5)
        feature_set.relatives[inp8] = 'Shift normal to helix select option'
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_micrograph_and_helix_selection_criteria(self, feature_set):
        feature_set = self.set_in_or_exclude_micrographs_option(feature_set)
        feature_set = self.set_in_or_exclude_micrographs(feature_set)
        feature_set = self.set_micrographs_to_be_in_or_excluded(feature_set)
         
        feature_set = self.set_in_or_exclude_helices_option(feature_set)
        feature_set = self.set_in_or_exclude_helices(feature_set)
        feature_set = self.set_helices_to_be_in_or_excluded(feature_set)
         
        return feature_set
    

    def set_defocus_and_astigmatism_selection_criteria(self, feature_set):
        feature_set = self.set_in_or_exclude_defocus_option(feature_set)
        feature_set = self.set_in_or_exclude_defocus(feature_set)
        feature_set = self.set_defocus_to_be_in_or_excluded(feature_set)

        feature_set = self.set_in_or_exclude_astigmatism_option(feature_set)
        feature_set = self.set_in_or_exclude_astigmatism(feature_set)
        feature_set = self.set_astigmatism_to_be_in_or_excluded(feature_set)

        return feature_set


    def set_curvature_selection_criteria(self, feature_set):
        feature_set = self.set_in_or_exclude_curvature_option(feature_set)
        feature_set = self.set_in_or_exclude_curvature(feature_set)
        feature_set = self.set_curvature_to_be_in_or_excluded(feature_set)

        return feature_set


    def set_selection_criteria_from_segment_table(self, feature_set):
        feature_set = self.set_micrograph_and_helix_selection_criteria(feature_set)
        
        feature_set = self.set_in_or_exclude_segments_option(feature_set)
        feature_set = self.set_in_or_exclude_segments(feature_set)
        feature_set = self.set_segments_from_segment_file_to_be_in_or_excluded(feature_set)

        feature_set = self.set_in_or_exclude_classes_option(feature_set)
        feature_set = self.set_in_or_exclude_classes(feature_set)
        feature_set = self.set_class_type(feature_set)
        feature_set = self.set_classes_to_be_in_or_excluded(feature_set)
        
        feature_set = self.set_persistence_class_selection_option(feature_set)
        feature_set = self.set_persistence_class_length(feature_set)
        feature_set = self.set_persistence_class_occupancy(feature_set)

        feature_set = self.set_curvature_selection_criteria(feature_set)
        
        feature_set = self.set_in_or_exclude_layer_ccc_option(feature_set)
        feature_set = self.set_in_or_exclude_ccc_layer(feature_set)
        feature_set = self.set_ccc_layer_to_be_in_or_excluded(feature_set)
        
        feature_set = self.set_defocus_and_astigmatism_selection_criteria(feature_set)
        
        return feature_set
    
    
    def set_selection_criteria_from_refined_segments_table(self, feature_set):
        feature_set = self.set_in_or_exclude_prj_ccc_option(feature_set)
        feature_set = self.set_in_or_exclude_ccc_prj_match(feature_set)
        feature_set = self.set_ccc_prj_match_to_be_in_or_excluded(feature_set)
        
        feature_set = self.set_in_or_exclude_out_of_plane_option(feature_set)
        feature_set = self.set_in_or_exclude_out_of_plane_tilt(feature_set)
        feature_set = self.set_out_of_plane_tilt_to_be_in_or_excluded(feature_set)
        
        feature_set = self.set_in_or_exclude_helix_shift_x_option(feature_set)
        feature_set = self.set_in_or_exclude_helix_shift_x(feature_set)
        feature_set = self.set_helix_shift_x_to_be_in_or_excluded(feature_set)
        
        return feature_set
    
    
    def set_database_selected(self, feature_set):
        inp1 = 'Selected database'
        feature_set.parameters[inp1] = 'spring_selected.db'
        feature_set.properties[inp1] = feature_set.file_properties(1, ['db'], 'saveFile')
        feature_set.hints[inp1] = 'Segment in this database will be reduced to selected properties. This database ' + \
        'can be used for selected refinement together with the selected output image stack.'
        feature_set.level[inp1]='beginner'
        
        return feature_set
    

class FeaturesStack(object):
    def set_inp_stack(self, feature_set):
        inp1 = 'Image input stack'
        feature_set.parameters[inp1]='protein_stack.hdf'
        feature_set.properties[inp1]=feature_set.file_properties(1,['hdf'],'getFile')
        feature_set.hints[inp1]='Input stack: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        
        return feature_set
        
        
    def set_class_avg_stack(self, feature_set):
        inp1 = 'Class average stack'
        feature_set.parameters[inp1] = 'avgerages.hdf'
        feature_set.properties[inp1] = feature_set.file_properties(1, ['hdf'], 'getFile')
        feature_set.hints[inp1] = 'Class average stack: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        
        return feature_set


    def set_class_number_to_be_analyzed(self, feature_set):
        inp7 = 'Class number to be analyzed'
        feature_set.parameters[inp7] = int(3)
        feature_set.hints[inp7] = 'Class number to be analyzed (1st class is 0).'
        feature_set.properties[inp7] = feature_set.Range(0, 2000, 1)
        feature_set.level[inp7]='beginner'

        return feature_set
    
    
    def set_class_number_range_to_be_analyzed(self, feature_set):
        inp7 = 'Class number range to be analyzed'
        feature_set.parameters[inp7] = tuple((int(3), int(5)))
        feature_set.hints[inp7] = 'Class number range to be analyzed (1st class is 0).'
        feature_set.properties[inp7] = feature_set.Range(0, 2000, 1)
        feature_set.level[inp7]='beginner'

        return feature_set
    
    
    def set_out_stack(self, feature_set):
        inp1 = 'Image output stack'
        feature_set.parameters[inp1]='protein_stack.hdf'
        feature_set.properties[inp1]=feature_set.file_properties(1,['hdf'],'saveFile')
        feature_set.hints[inp1]='Output stack: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        
        return feature_set


    def set_reference_option(self, feature_set):
        inp5 = 'Reference image option'
        feature_set.parameters[inp5]=bool(False)
        feature_set.hints[inp5]='If reference stack desired to continue classification.'
        feature_set.level[inp5]='intermediate'
        
        return feature_set
    
    
    def set_image_reference_stack(self, feature_set, current_level='beginner', relative=None):
        inp1 = 'Image reference stack'
        feature_set.parameters[inp1] = 'protein_reference_stack.hdf'
        feature_set.properties[inp1] = feature_set.file_properties(1, ['spi', 'hdf', 'img', 'hed'], 'getFile')
        feature_set.hints[inp1] = 'Input reference stack: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]=current_level
        if relative is not None:
            feature_set.relatives[inp1]=relative
        
        return feature_set


    def set_reference_update_option(self, feature_set):
        inp1 = 'Update references'
        feature_set.parameters[inp1] = True
        feature_set.hints[inp1] = 'Check if references should be updated for every iteration. This is useful if ' + \
        'you have a high ratio of images / references and you expect to improve your references. In case you ' + \
        'simply want to match or assign having a low images / references ratio you should turn it off.'
        
        feature_set.level[inp1]='beginner'
        
        return feature_set
    
    
    def set_reference_output_aligned(self, feature_set):
        inp1 = 'Aligned average stack'
        feature_set.parameters[inp1] = 'protein_reference_aligned.hdf'
        feature_set.properties[inp1] = feature_set.file_properties(1, ['spi', 'hdf', 'img', 'hed'], 'saveFile')
        feature_set.hints[inp1] = 'Output aligned reference stack: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp1)
        
        feature_set.level[inp1]='beginner'
        feature_set.relatives[inp1]='Update references'
        
        return feature_set
    
    
class FeaturesDatabase(object):
    def set_spring_db_option(self, feature_set, current_level='intermediate', default=True, relative=None):
        inp6 = 'Spring database option'
        feature_set.parameters[inp6] = bool(default)
        feature_set.hints[inp6] = 'If checked will read previous spring.db (Sqlite-compatible database) otherwise ' + \
        'will create new one.'
        feature_set.level[inp6]=current_level
        if relative is not None:
            feature_set.relatives[inp6]=relative 
        
        return feature_set

    
    def set_spring_path(self, feature_set, current_level='intermediate', relative='Spring database option'):
        inp3 = 'spring.db file'
        feature_set.parameters[inp3]='spring.db'
        feature_set.hints[inp3]='Program requires a previously generated spring.db and writes an updated spring.db ' + \
        'database in the working directory.'
        feature_set.properties[inp3]=feature_set.file_properties(1,['db'],'getFile')
        feature_set.relatives[inp3]=relative
        feature_set.level[inp3]=current_level
        
        return feature_set
    

    def set_spring_path_segments(self, feature_set):
        inp3 = 'spring.db file'
        feature_set.parameters[inp3]='spring.db'
        feature_set.hints[inp3]='Program requires a combined spring.db from segment (and optional: ' + \
        'micctfdetermine). An updated spring.db will be created in the working directory.'
        feature_set.properties[inp3]=feature_set.file_properties(1,['db'],'getFile')
        feature_set.level[inp3]='beginner'
        
        return feature_set
    
    
    def set_inp_refinement_path(self, feature_set):
        inp3 = 'refinement.db file'
        feature_set.parameters[inp3]='refinement.db'
        feature_set.hints[inp3]='Requires refinement.db from segmentrefine3d to extract refinment parameters.'
        feature_set.properties[inp3]=feature_set.file_properties(1,['db'],'getFile')
        feature_set.level[inp3]='beginner'
        
        return feature_set
    
    
    def set_grid_database(self, feature_set, singular=True):
        inp9 = 'Grid database'
        feature_set.parameters[inp9]='grid.db'
        if singular:
            file_attr = 'getFile'
        else:
            file_attr = 'getFiles'
        feature_set.properties[inp9]=feature_set.file_properties(1,['db'], file_attr)
        feature_set.hints[inp9]='Grid database with associated metadata from segclassreconstruct or segrefine3dgrid: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp9)
        
        feature_set.level[inp9]='beginner'
        
        return feature_set
    
    
    def set_continue_refinement_option(self, feature_set):
        inp5 = 'Continue refinement option'
        feature_set.parameters[inp5]=bool(False)
        feature_set.hints[inp5]='To continue previous refinement and use final alignment parameters as ' + \
        'starting point. In case no reference structure is provided, it will re-create the final 3D reconstruction ' + \
        'without the projection matching step. When a reference structure is provided, it will perform ' + \
        'projection matching and 3D reconstruction.'
        feature_set.level[inp5]='intermediate'
        
        return feature_set
    
    
    def set_refinementdb_path(self, feature_set):
        inp3 = 'refinement.db file'
        feature_set.parameters[inp3]='refinement.db'
        feature_set.hints[inp3]='Program requires a refinement.db from previous segmentrefine3d. An updated ' + \
        'refinement.db will be created in the working directory.'
        feature_set.properties[inp3]=feature_set.file_properties(1,['db'],'getFile')
        feature_set.relatives[inp3]='Continue refinement option'
        feature_set.level[inp3]='intermediate'
        
        return feature_set
    
    
class FeaturesGrid(object):
    def set_first_parameter_choice(self, feature_set, parameters_to_be_varied, default='helical_rise_or_pitch'):
        inp7 = 'First parameter'
        feature_set.parameters[inp7] = str(default)
        parameters = ''.join(['\'{0}\'\n'.format(each_parameter) for each_parameter in parameters_to_be_varied])
        feature_set.hints[inp7] = 'Choose parameter to be varied in first dimension: \n{0}'.format(parameters)
        feature_set.properties[inp7] = feature_set.choice_properties(2, parameters_to_be_varied, 'QComboBox')
        feature_set.level[inp7]='beginner'

        return feature_set
    
    
    def set_second_parameter_choice(self, feature_set, parameters_to_be_varied,
    default='helical_rotation_or_number_of_units_per_turn'):
        inp7 = 'Second parameter'
        feature_set.parameters[inp7] = str(default)
        parameters_to_be_varied.insert(0, 'none')
        parameters = ''.join(['\'{0}\'\n'.format(each_parameter) for each_parameter in parameters_to_be_varied])
        feature_set.hints[inp7] = 'Choose parameter to be varied in second dimension: \n{0}'.format(parameters)
        feature_set.properties[inp7] = feature_set.choice_properties(2, parameters_to_be_varied, 'QComboBox')
        feature_set.level[inp7]='beginner'

        return feature_set
    


    def get_unit_dependence_hint(self):
        return 'Unit dependent on quantity.'

    def set_lower_and_upper_limit_first_parameter(self, feature_set):
        inp8 = 'Lower and upper limit first parameter'
        feature_set.parameters[inp8] = tuple((1.4, 1.9))
        
        feature_set.hints[inp8] = 'Lower and upper limit of first parameter for grid search. ' + \
        self.get_unit_dependence_hint()
        
        feature_set.properties[inp8] = feature_set.Range(-10000000, 10000000, 0.001)
        feature_set.level[inp8]='beginner'
        
        return feature_set

    
    def set_lower_and_upper_limit_second_parameter(self, feature_set):
        inp8 = 'Lower and upper limit second parameter'
        feature_set.parameters[inp8] = tuple((22.0, 24.0))
        
        feature_set.hints[inp8] = 'Lower and upper limit of second parameter for grid search. ' + \
        self.get_unit_dependence_hint()
        
        feature_set.properties[inp8] = feature_set.Range(-10000000, 10000000, 0.001)
        feature_set.level[inp8]='beginner'
        
        return feature_set
    
    
    def set_first_and_second_parameter_increment(self, feature_set):
        inp8 = 'First and second parameter increment'
        feature_set.parameters[inp8] = tuple((0.1, 0.3))
        
        feature_set.hints[inp8] = 'First and second parameter increment for grid search. ' + \
        self.get_unit_dependence_hint()
        
        feature_set.properties[inp8] = feature_set.Range(-1e8, 1e+8, 0.001)
        feature_set.level[inp8]='beginner'
        
        return feature_set
    
    
    def set_subgrid_option(self, feature_set):
        inp8 = 'Subgrid option'
        feature_set.parameters[inp8] = False
        feature_set.hints[inp8] = 'Run subgrids to parallelize expensive grid searches.'
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_subgrid_details(self, feature_set):
        inp8 = 'Part and number of subgrids'
        feature_set.parameters[inp8] = tuple((1, 3))
        feature_set.hints[inp8] = 'E.g. one out of total of three subgrids will be run. This features is thought for '+\
        'parallelization of expensive grid searches.'
        feature_set.properties[inp8] = feature_set.Range(1, 100, 1)
        feature_set.relatives[inp8]=('Subgrid option', 'Subgrid option')
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_continue_grid_option(self, feature_set):
        inp8 = 'Grid continue option'
        feature_set.parameters[inp8] = False
        feature_set.hints[inp8] = 'Continue grid refinement in case of interrupted grid searches.'
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
    def set_grid_database_cont(self, feature_set):
        inp8 = 'Grid database'
        feature_set.parameters[inp8] = 'grid.db'
        feature_set.properties[inp8]=feature_set.file_properties(1,['db'],'getFile')
        feature_set.hints[inp8] = 'Continue grid refinement in case of interrupted grid searches.'
        feature_set.relatives[inp8]='Grid continue option'
        feature_set.level[inp8]='expert'
        
        return feature_set
    
    
class FeaturesConvert(object):
    def convert_list_of_numbers_from_entry_string(self, entry, entry_type=int):
        """
        >>> from spring.csinfrastr.csfeatures import Features
        >>> f = Features()
        >>> f.convert_list_of_numbers_from_entry_string('1, 1, 10')
        [1, 1, 10]
        >>> f.convert_list_of_numbers_from_entry_string('1, 1, a', entry_type=float)
        Traceback (most recent call last):
            ...
        ValueError: Entry 'a' could not be interpreted with expected number: <type 'float'>
        """
        converted = []
        for each_entry in entry.split(','):
            try:
                converted.append(entry_type(each_entry.strip()))
            except:
                msg = 'Entry \'{0}\' could not be interpreted with expected number: {1}'.format(each_entry.strip(),
                entry_type)

                raise ValueError(msg)

        return converted
        

    def convert_list_of_specific_strings_from_entry_string(self, entry, quantities=['polar', 'apolar']):
        """
        >>> from spring.csinfrastr.csfeatures import Features
        >>> f = Features()
        >>> f.convert_list_of_specific_strings_from_entry_string('polar, apolar, polar')
        ['polar', 'apolar', 'polar']
        >>> f.convert_list_of_specific_strings_from_entry_string('opolar, apolar, polar')
        Traceback (most recent call last):
            ...
        ValueError: Entry 'opolar' could not be matched with expected: polar, apolar
        """
        converted = []
        for each_entry in entry.split(','):
            if each_entry.strip() in quantities:
                converted.append(each_entry.strip())
            else:
                msg = 'Entry \'{0}\' could not be matched with expected: {1}'.format(each_entry.strip(), 
                ", ".join(quantities))
                raise ValueError(msg)
        
        return converted

    def convert_list_of_data_pairs_from_entry_string(self, entry, quantities=('Layer line position (1/Angstrom)', 
        'Bessel order')):
        """
        >>> from spring.csinfrastr.csfeatures import Features
        >>> f = Features()
        >>> first = '(0.1, -2)'
        >>> entry = first + '; (0.2, 4); (0.3, -6)'
        >>> f.convert_list_of_data_pairs_from_entry_string(first)
        [(0.1, -2)]
        >>> f.convert_list_of_data_pairs_from_entry_string('(10, 50);(20, 80);(30, 30)')
        [(10.0, 50), (20.0, 80), (30.0, 30)]
        >>> f.convert_list_of_data_pairs_from_entry_string(entry)
        [(0.1, -2), (0.2, 4), (0.3, -6)]
        >>> wrong_entry = first + '; (0.2, 4); (0.3, -6g)'
        >>> f.convert_list_of_data_pairs_from_entry_string(wrong_entry) #doctest: +NORMALIZE_WHITESPACE
        Traceback (most recent call last):
            ...
        ValueError: No pairs with comma-separated numbers could be found. 
            Please comply with input format: (Layer line position (1/Angstrom), 
            Bessel order), e.g. '(0.1, -2); (0.2, -3)'
        """
        first_q, second_q = quantities
        error_message = 'No pairs with comma-separated numbers could be found. Please comply with input format: ' + \
        '({0}, {1}), e.g. \'(0.1, -2); (0.2, -3)\''.format(first_q, second_q)

        if entry.find(',') >= 0:
            pass
        else:
            raise ValueError(error_message)
            
        layerline_bessel_pair = []
        isolated_pairs = entry.split(';')
        try:
            for each_pair in isolated_pairs:
                pair = each_pair.strip().strip(')').strip('(')
                layerline_str, bessel_str = pair.split(',')
                if second_q == 'Bessel order':
                    layerline_bessel_pair.append((float(layerline_str), int(bessel_str)))
                else:
                    layerline_bessel_pair.append((float(layerline_str), float(bessel_str)))
        except:
            raise ValueError(error_message)
            
        return layerline_bessel_pair
            
        
    def convert_list_of_files_from_entry_string(self, inputfiles, check_location=True):
        """
        >>> from spring.csinfrastr.csfeatures import Features
        >>> f = Features()
        >>> f.convert_list_of_files_from_entry_string('*box')
        Traceback (most recent call last):
        ...
        ValueError: No files were found in the specified location: "*box". Provide correct file input.
        >>> input_files = ['1.box', '2.box', '3.box', '4.box']
        >>> fl = f.convert_list_of_files_from_entry_string(",".join(input_files))
        >>> fl = [os.path.basename(each_file) for each_file in fl]
        >>> assert fl == input_files
        >>> fl = f.convert_list_of_files_from_entry_string(";".join(input_files))
        >>> fl = [os.path.basename(each_file) for each_file in fl]
        >>> assert fl == input_files
        """
        if inputfiles.find('?') >= 0 or inputfiles.find('*') >= 0 or glob(inputfiles) != []:
            inputlist = glob(inputfiles) 
            inputlist.sort()
            if inputlist == [] and check_location:
                msg = 'No files were found in the specified location: \"{0}\". Provide correct file input.'.format(inputfiles)
                raise ValueError(msg)
        elif inputfiles.find(',') >= 0 :
            inputlist = inputfiles.split(',')
        elif inputfiles.find(';') >= 0 :
            inputlist = inputfiles.split(';')
        else:
            inputlist = [inputfiles]
 
        def getabsfilename(infile):
            try:
                absfile = os.path.abspath(infile)
            except OSError:
                errstring = 'File {0} does not exist'.format(infile)
                raise IOError(errstring)
            return absfile

        inputabslist = [ getabsfilename(infile.strip()) for infile in inputlist]

        return inputabslist
    
    
class Features(FeaturesBinning, FeaturesFilter, FeaturesAlign, FeaturesHelix3dReconstruction, FeaturesMicrograph,
FeaturesParticle, FeaturesPower, FeaturesInteractive, FeaturesMpi, FeaturesSelection, FeaturesStack, FeaturesDatabase,
FeaturesGrid, FeaturesConvert):
    """
    * Class that initiates ...Par classes
    """
    def setup(self, sett):
        """
        * Function to set up dictionaries for each program, including parameter, help, range and status information

        """

        # define default input dictionary p, help dictionary h, range dictionary r, status dictionary s
        sett.parameters = OrderedDict()
        sett.hints = OrderedDict()
        sett.properties = OrderedDict() 
        sett.relatives = OrderedDict()
        sett.level = OrderedDict()
        sett.program_states = OrderedDict()
        sett.Range = namedtuple('Range', 'minimum maximum step')
        sett.file_properties = namedtuple('file_properties', 'number ext ftype')
        sett.choice_properties = namedtuple('choice_properties', 'number choices ctype')

        # retrieve metadata from EGG
        sett.eggmeta = GetMetaData(sett.package).release_meta()
        sett.version = sett.eggmeta['Version']

        return sett

    def reduce_features_to_parameters_for_mpi(self, parameters):
        self.parameters = parameters
        
        return self
    


    def get_parameters_in_string(self, parameters):
        par_string = ''
        for name in parameters:
            if parameters[name] is not None:
                par_string += '%-40s = %s\n' % (name, parameters[name])
        
        return par_string

    def write_parameters_in_file(self, parameters, prgparfile):
        parfile = open(prgparfile, 'w')
        
        par_string = self.get_parameters_in_string(parameters)
        
        parfile.write(par_string)
        parfile.close()


    def get_submission_script_str(self, program, parameters):
        header = """\
#!/usr/bin/env python

import subprocess
import time
import os

if 'PBS_O_WORKDIR' in os.environ:
    os.chdir(os.environ['PBS_O_WORKDIR'])

program = '{0}'
par_file_content = \'\'\'
""".format(program)

        par_string = self.get_parameters_in_string(parameters) + '\'\'\''
        
        launch_lines = """\

dir_name = '{prg}_{zeit}_{pid}'.format(prg=os.path.split(program)[-1], zeit=time.strftime('%d_%b_%Y_%H_%M_%S'), \
pid=os.getpid())

par_file_name = '{dir_name}.par'.format(dir_name=dir_name)
f = open(par_file_name, 'w')
f.write(par_file_content)
f.close()

command_line = '{prg} --f {par_file} --d {dir_name}'.format(prg=program, par_file=par_file_name, dir_name=dir_name)
print command_line

subprocess.call(command_line.split())
os.rename(par_file_name, dir_name + os.sep + 'parameters.par')
"""
        return header + par_string + launch_lines

        
    def write_parfile(self, parameters=None, program=None, tag=True):

        if program is None:
            prgparfile = 'parfile_{0}.par'.format(os.getpid())
        else:
            prgparfile = '{prg}_{zeit}_{id}.par'.format(prg=program, zeit=time.strftime('%d_%b_%Y_%H_%M_%S'),
            id=os.getpid())
        if not tag:
            prgparfile = '{prg}.par'.format(prg=program) 
            
        self.write_parameters_in_file(parameters, prgparfile)

        return prgparfile


    def rename_series_of_output_files(self, micrograph_files, outfile):
        """
        >>> from spring.csinfrastr.csfeatures import Features
        >>> f = Features()
        >>> f.rename_series_of_output_files(['test0.mrc'], 'dp.pdf')
        ['dp.pdf']
        >>> f.rename_series_of_output_files(['test0.mrc','test1.mrc'], 'dp.pdf')
        ['dp_test0.pdf', 'dp_test1.pdf']
        """
        if len(micrograph_files) == 1:
            outfiles = [outfile]
        else:
            outfiles = [os.path.splitext(os.path.basename(outfile))[0] + '_' +
            os.path.splitext(os.path.basename(each_micrograph_file))[0] + os.path.splitext(outfile)[-1] for
            each_micrograph_file in micrograph_files]
            
        return outfiles
    

    def set_output_plot(self, feature_set, output, related=None):
        inp2 = 'Diagnostic plot'
        feature_set.parameters[inp2]=str(output)
        
        feature_set.properties[inp2]=feature_set.file_properties(1,
        FeaturesSupport().get_diagnostic_output_formats_for_matplotlib(), 'saveFile')
        
        feature_set.hints[inp2]='Output diagnostic plot: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp2)
        
        feature_set.level[inp2]='beginner'
        if related is not None:
            feature_set.relatives[inp2]=related
        
        return feature_set
    
    
    def set_output_plot_pattern(self, feature_set, output):
        inp2 = 'Diagnostic plot pattern'
        feature_set.parameters[inp2]=str(output)
        
        feature_set.properties[inp2]=feature_set.file_properties(1, 
        FeaturesSupport().get_diagnostic_output_formats_for_matplotlib(), 'saveFile')
        
        feature_set.hints[inp2]='If single input micrograph: name of diagnostic plot file. In case of multiple ' + \
        'input micrographs suffix to be attached to corresponding input micrograph. Output: ' + \
        FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp2)
        
        feature_set.level[inp2]='beginner'
        
        return feature_set
    
    
    def set_diagnostic_prefix(self, feature_set, level='beginner', related=None):
        inp3 = 'Diagnostic plot prefix'
        feature_set.parameters[inp3]='diagnostic_plot.pdf'
        feature_set.properties[inp3]=feature_set.file_properties(1, 
        FeaturesSupport().get_diagnostic_output_formats_for_matplotlib(), 'saveFile')
        
        feature_set.hints[inp3]='Output name for diagnostic plots of iterative structure refinement (completion ' + \
        'to \'prefix_XXX.ext\'): ' + FeaturesSupport().add_accepted_file_formats_to_hint(feature_set, inp3) 
        feature_set.level[inp3]=level
        if related is not None:
            feature_set.relatives[inp3]=related
        
        return feature_set
    
    
    def set_temppath(self, feature_set):
        inp3 = 'Temporary directory'
        feature_set.parameters[inp3]='/tmp'
        feature_set.hints[inp3]='Temporary directory should have fast read and write access.'
        feature_set.properties[inp3]=feature_set.file_properties(1,['*'],'getDir')
        feature_set.level[inp3]='intermediate'
        
        return feature_set


    def set_scanner_step_size(self, feature_set):
        inp3 = 'Scanner step size in micrometer' 
        feature_set.parameters[inp3] = float(7.0)
        feature_set.hints[inp3] = 'Characteristic resolution or step size of scanner in micrometer.'
        feature_set.properties[inp3] = feature_set.Range(1, 1000, 1)
        feature_set.level[inp3] = 'intermediate'
        
        return feature_set




# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from EMAN2 import Util, Transform
from collections import namedtuple
from fundamentals import rot_shift2D, prepi
from scipy import interpolate
from spring.csinfrastr.csdatabase import SpringDataBase, base, CtfMicrographTable, SegmentTable, HelixTable
from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment2d.segment_prep import SegmentPreparationFileRead
from tabulate import tabulate
from utilities import model_blank
import numpy as np
import os
    
                        
class SegmentInterpolation(SegmentPreparationFileRead):
    def make_helixinfo_named_tuple(self):
        helixinfo = namedtuple('helixinfo', 'helix_id micrograph segment_list directory coordinates ' + \
                                            'picked_coordinates inplane_angle curvature')
        
        return helixinfo


    def convert_coordinate_line_to_xy_coordinates(self, each_coordinate_line):
        col = each_coordinate_line.split()
        xcoord = float(col[0])
        ycoord = float(col[1])
        boxsize = float(col[2])
        xcenter = xcoord + boxsize / 2
        ycenter = ycoord + boxsize / 2
        
        return xcenter, ycenter
    

    def compute_cumulative_distances_from_start_of_helix(self, x_coordinates, y_coordinates):
        """
        >>> from spring.segment2d.segment import Segment
        >>> x_coordinates = y_coordinates = np.arange(10)
        >>> s = Segment()
        >>> s.compute_cumulative_distances_from_start_of_helix(x_coordinates, y_coordinates)
        array([  0.        ,   1.41421356,   2.82842712,   4.24264069,
                 5.65685425,   7.07106781,   8.48528137,   9.89949494,
                11.3137085 ,  12.72792206])
        >>> x_coordinates = 10 * np.ones(10)
        >>> y_coordinates = np.arange(10)
        >>> s.compute_cumulative_distances_from_start_of_helix(x_coordinates, y_coordinates)
        array([ 0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9.])
        """
        previous_x_coord = np.roll(np.array(x_coordinates), 1)
        previous_y_coord = np.roll(np.array(y_coordinates), 1)
        distances = np.sqrt((x_coordinates - previous_x_coord)**2 + (y_coordinates - previous_y_coord)**2)
        distances[0] = 0
        distances_from_start = np.cumsum(distances)
        
        return distances_from_start

    def enter_helixinfo_into_helices_and_write_boxfiles(self, helices, micfile, overlap, helixfile, ipangle, curvature,
    picked_coordinates, int_xy_coordinates):
        total_helix_number = len(helices) + 1
        helices = self.enter_helixinfo_into_helices(helices, micfile, overlap, helixfile, ipangle, curvature,
                                                    picked_coordinates, int_xy_coordinates)

        interpolated_xcoord, interpolated_ycoord = zip(*int_xy_coordinates)

        self.write_boxfile(np.array(interpolated_xcoord), np.array(interpolated_ycoord), self.segsizepix,
        filename=helixfile)

        data = zip([total_helix_number] + (len(int_xy_coordinates) - 1) * [None], 
            [os.path.basename(helixfile)] + (len(int_xy_coordinates) - 1) * [None], 
            interpolated_xcoord, interpolated_ycoord)
        log_info = '\n' + tabulate(data, ['helix_id', 'helix_file', 'x-coordinate', 'y-coordinate'])
        self.log.ilog(log_info)
        
        return helices

    def enter_helixinfo_into_helices(self, helices, micfile, overlap, helixfile, ipangle, curvature, picked_coordinates,
                                     int_xy_coordinates):
        helixinfo = self.make_helixinfo_named_tuple()
        total_helix_number = len(helices) + 1

        helices.append(helixinfo(total_helix_number, micfile, helixfile, overlap, int_xy_coordinates,
                                 picked_coordinates, ipangle, curvature))
        return helices


    def perform_interpolation_perturbation_removal_if_required(self, helices, segfile, stepsize, pixelsize, helixwidth,
    micfile, overlap, helixfile, x_coordinates, y_coordinates):
        
        if self.frame_option:
            interpolated_xcoord, interpolated_ycoord, ipangle, curvature = \
            self.interpolate_coordinates(np.array(x_coordinates), np.array(y_coordinates), pixelsize, stepsize, helixwidth,
            segfile, new_stepsize=False)
        
            helices = self.enter_helixinfo_into_helices_and_write_boxfiles(helices, micfile, overlap, helixfile,
            ipangle, curvature, list(zip(interpolated_xcoord, interpolated_ycoord)), list(zip(interpolated_xcoord,
            interpolated_ycoord)))
        else:
            interpolated_xcoord, interpolated_ycoord, ipangle, curvature = \
            self.interpolate_coordinates(np.array(x_coordinates), np.array(y_coordinates), pixelsize, stepsize, helixwidth,
            segfile, new_stepsize=True)
            
            if self.remove_ends and stepsize != 0:
                remove_end_count = int(round(0.5 * self.segment_size / stepsize))
                
                interpolated_xcoord = interpolated_xcoord[remove_end_count:-remove_end_count]
                interpolated_ycoord = interpolated_ycoord[remove_end_count:-remove_end_count]
                ipangle = ipangle[remove_end_count:-remove_end_count]
                curvature = curvature[remove_end_count:-remove_end_count]
            picked_coordinates = list(zip(interpolated_xcoord, interpolated_ycoord))
            
            if self.perturb_step:
                rand_y_shifts = 0.9 * stepsize / pixelsize * np.array([(np.random.random() - 0.5) for 
                        each_xcoord in interpolated_xcoord[1:-1]])
                centers_x = interpolated_xcoord[:-2]
                centers_y = interpolated_ycoord[:-2]
                hx, hy = self.rotate_coordinates_by_angle(interpolated_xcoord[1:-1], 
                    interpolated_ycoord[1:-1], ipangle[1:-1], centers_x, centers_y)
                hy += rand_y_shifts
                hx, hy = self.rotate_coordinates_by_angle(hx, hy, -ipangle[1:-1], centers_x, centers_y)
                interpolated_xcoord[1:-1] = hx
                interpolated_ycoord[1:-1] = hy
            int_xy_coordinates = list(zip(interpolated_xcoord, interpolated_ycoord))
            
            if len(int_xy_coordinates) >= 3:
                helices = self.enter_helixinfo_into_helices_and_write_boxfiles(helices, micfile, overlap, helixfile,
                ipangle, curvature, picked_coordinates, int_xy_coordinates)
            
        return helices
    

    def fill_in_helix_info_from_coordinates(self, helices, segfile, stepsize, pixelsize, helixwidth, micfile, overlap,
    mic_hindex, x_coordinates, y_coordinates):
        helixfile = '{0}{1}{2}_{3:03}{4}box'.format(os.path.abspath(overlap), os.sep,
        os.path.basename(os.path.splitext(segfile)[0]), mic_hindex, os.extsep)

        helix_length = pixelsize * np.sqrt(
            (y_coordinates[-1] - y_coordinates[0]) ** 2 + (x_coordinates[-1] - x_coordinates[0]) ** 2)

        if helix_length < 3 * stepsize:
            log_info = 'Helix from {0} was not further used because length of helix is '.format(helixfile) + \
            'shorter than specified 3 * stepsize ' + \
            '({0} Angstrom < {1} Angstrom). '.format(helix_length, 3 * stepsize) + \
            'No sensible segmentation possible. '

            self.log.ilog(log_info)
        else:
            helices = self.perform_interpolation_perturbation_removal_if_required(helices, segfile, stepsize, pixelsize,
            helixwidth, micfile, overlap, helixfile, x_coordinates, y_coordinates)

        return helices


    def compute_new_coordinates_with_given_stepsize(self, helices, segfile, stepsize, pixelsize, helixwidth, 
        micfile, overlap, helixcoord):
        
        mic_hindex = 0
        for helix in helixcoord:
            xycentercoord = []
            for each_coordinate_line in helix.splitlines():
                xcenter, ycenter = self.convert_coordinate_line_to_xy_coordinates(each_coordinate_line)
                xycentercoord.append((xcenter, ycenter))
                if each_coordinate_line.endswith('-2'):
                    x_coordinates, y_coordinates = list(zip(*xycentercoord))
                    
                    helices = self.fill_in_helix_info_from_coordinates(helices, segfile, stepsize, pixelsize,
                    helixwidth, micfile, overlap, mic_hindex, x_coordinates, y_coordinates)
            
                    mic_hindex += 1

        return helices
    

    def get_picked_coordinates_from_database(self, micfile, db_name, session):
        if session is None:
            session = SpringDataBase().setup_sqlite_db(base, db_name)

        this_mic = session.query(CtfMicrographTable).\
        filter(CtfMicrographTable.micrograph_name.startswith(os.path.basename(os.path.splitext(micfile)[0]))).first()
        
        helices = session.query(HelixTable).filter(HelixTable.mic_id == this_mic.id).order_by(HelixTable.id)
        mic_coord = []
        for each_helix in helices:
            segments = session.query(SegmentTable).\
            filter(SegmentTable.helix_id == each_helix.id).all()

            coordinates = [(each_segment.picked_x_coordinate_A / self.pixelsize, 
            each_segment.picked_y_coordinate_A / self.pixelsize) for each_segment in segments]
            
            mic_coord.append(coordinates)
            
        return session, mic_coord

        
    def get_picked_coordinates_from_file(self, segfile, overlap):
        ifile = open(os.path.abspath(overlap) + os.sep + os.path.basename(segfile), 'r')
        coordinate_lines = ifile.readlines()
        ifile.close()
        if segfile.endswith('box') or segfile.endswith('txt'):
            helixcoord = self.read_coordinate_lines_from_eman_format_and_separate_helices(coordinate_lines)
        elif segfile.endswith('star'):
            helixcoord = \
            self.read_coordinate_lines_from_bsoft_particle_format_and_convert_to_eman_format(coordinate_lines)
            
            if helixcoord == []:
                helixcoord = \
                self.read_coordinate_lines_from_bsoft_filament_format_and_convert_to_eman_format(coordinate_lines)
                
        return helixcoord
    

    def assign_ids_based_on_new_frames(self, inp_stack_ids, inp_mics, mic_coord):
        """
        >>> from spring.segment2d.segment import Segment
        >>> s = Segment()
        >>> mic_coord = [zip(np.arange(3), np.arange(3)), zip(np.arange(5), np.arange(5))]
        >>> first = s.assign_ids_based_on_new_frames([], list(range(5)), mic_coord)
        >>> first #doctest: +NORMALIZE_WHITESPACE
        [0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7,
        0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7]
        >>> s.assign_ids_based_on_new_frames(first, list(range(5)), mic_coord) #doctest: +NORMALIZE_WHITESPACE
        [0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 
        0, 1, 2, 3, 4, 5, 6, 7, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 
        15, 8, 9, 10, 11, 12, 13, 14, 15, 8, 9, 10, 11, 12, 13, 14, 15, 8, 9, 10, 
        11, 12, 13, 14, 15, 8, 9, 10, 11, 12, 13, 14, 15]
        >>> helices = [1, 2]
        >>> s.assign_ids_based_on_new_frames([], list(range(5)), helices)
        [0, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        """
        all_coord = np.vstack(mic_coord)
        stack_ids = []
        if inp_stack_ids == []:
            max_inp_stack_id = -1
        else:
            max_inp_stack_id = max(inp_stack_ids)

        for each_mic_id, each_inp in enumerate(inp_mics):
            for each_coord_id, each_coord in enumerate(all_coord):
                stack_ids.append(each_coord_id + max_inp_stack_id + 1)
            
        if len(inp_stack_ids) == 0:
            merged_stack_ids = stack_ids
        else:
            merged_stack_ids = inp_stack_ids + stack_ids

        return merged_stack_ids
                    
                
    def single_out(self, pair, stepsize, pixelsize, assigned_mics):
        """ 
        * Function to single out individual helices from entire micrograph, re-interpolates coordinates\
            according to specified stepsize, 

        #. Input: list of 3-tuples (micrograph file, segment file, overlapping letters), stepsize, pixelsize
        #. Output: list of helices containing micrograph, segment_list, directory, coordinates, inplane_angle
        #. Usage: self.helices = single_out(pair, stepsize, pixelsize)
        """
        self.log.fcttolog()

        helices = []
        session = None
        if assigned_mics is not None:
            assigned_mics = dict(assigned_mics)

        assigned_stack_ids = []
        assigned_helix_ids = []
        for (micfile, segfile, overlap) in pair:
            if segfile.endswith('db'):
                session, mic_coord = self.get_picked_coordinates_from_database(micfile, segfile, session)

                base_micfile = os.path.basename(micfile)
                if assigned_mics is not None:
                    inp_mics = assigned_mics[base_micfile]
                else:
                    inp_mics = [micfile]

                assigned_stack_ids = self.assign_ids_based_on_new_frames(assigned_stack_ids, inp_mics, mic_coord)

                assigned_helix_ids = self.assign_ids_based_on_new_frames(assigned_helix_ids, inp_mics, 
                list(range(len(mic_coord))))

                for each_inp_mic in inp_mics:
                    for mic_hindex, each_coord in enumerate(mic_coord):
                        segfile = os.path.splitext(each_inp_mic)[0] + os.extsep + 'box'
                        each_x_coord, each_y_coord = zip(*each_coord)

                        helixfile = '{0}{1}{2}_{3:03}{4}box'.format(os.path.abspath(overlap), os.sep,
                        os.path.basename(os.path.splitext(segfile)[0]), mic_hindex, os.extsep)

                        helices = self.perform_interpolation_perturbation_removal_if_required(helices, segfile,
                        stepsize, pixelsize, self.helixwidth, each_inp_mic, overlap, helixfile, each_x_coord,
                        each_y_coord)
            else:
                helixcoord = self.get_picked_coordinates_from_file(segfile, overlap)
            
                helices = self.compute_new_coordinates_with_given_stepsize(helices, segfile, stepsize,
                pixelsize, self.helixwidth, micfile, overlap, helixcoord)
                
                assigned_stack_ids = None
                                                                                  
        if helices == []:
            msg = 'No helices were segmented because picked helices were too short to be segmented. You ' + \
            'can provide longer helices, choose a shorter step size and/or disable the \'Remove helix ends option \''
            raise ValueError(msg)
            
        return helices, assigned_stack_ids, assigned_helix_ids


    def prepare_segmentation(self):
        OpenMpi().setup_and_start_mpi_version_if_demanded(self.mpi_option, self.feature_set, self.cpu_count)
        assigned_mics = self.validate_input()
                
        pair = self.assign_reorganize(self.micrograph_files, self.coordinate_files)

        self.helices, assigned_stack_ids, assigned_helix_ids = self.single_out(pair, self.stepsize, self.pixelsize,
        assigned_mics)
        
        return self.helices, assigned_stack_ids, assigned_helix_ids


    def determine_angle_to_rotate_coordinates_to_minimal_slope(self, xcoord, ycoord, helixwidth):
        if np.std(xcoord) < helixwidth/3 or np.std(ycoord) < helixwidth/3:
            if np.std(xcoord) < np.std(ycoord):
                mangle = 90.0
            else: 
                mangle = 0 
        else:
            polyline = np.polyfit((xcoord[0], xcoord[-1]), [ycoord[0], ycoord[-1]], 1)
#            self.log.dlog('xcoord: {0}, ycoord: {1}, polyline: {2}'.format(xcoord, ycoord, polyline[-2]))
            mangle = np.rad2deg(np.arctan(polyline[-2]))

        return mangle


    def rotate_coordinates_by_angle(self, xcoord, ycoord, angle_in_deg, center_x=0, center_y=0):
        """
        * Function to rotate coordinates clockwise by angle_in_deg around origin\
         (in correspondence to EMAN2 convention)

        #. Input: x- and y-array, angle_in_deg to be rotated
        #. Output: rotated x- and y-array
        #. Usage: rotx, roty = rotate_coordinates_by_angle(xcoord, ycoord, angle_in_deg)

        >>> import numpy as np
        >>> np.arange(0, 10)
        array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
        >>> np.zeros(10)
        array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.])
        >>> from spring.segment2d.segment import Segment
        >>> s = Segment()
        >>> rot_x, rot_y = s.rotate_coordinates_by_angle(np.arange(10), np.zeros(10), -90.0) 
        >>> np.rint(rot_x)
        array([ 0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.,  0.])
        >>> np.rint(rot_y)
        array([ 0.,  1.,  2.,  3.,  4.,  5.,  6.,  7.,  8.,  9.])
        >>> s.rotate_coordinates_by_angle(np.array([5]), np.array([0]), 180)
        (array([-5.]), array([ -6.12323400e-16]))
        >>> s.rotate_coordinates_by_angle(0, 0, 180, 4, 3)
        (8.0, 6.0)
        """
        c_xcoord = xcoord - center_x
        c_ycoord = ycoord - center_y
        
        angle_in_rad = np.deg2rad(angle_in_deg)
        rotx = np.cos(angle_in_rad)*c_xcoord + np.sin(angle_in_rad)*c_ycoord
        roty = -np.sin(angle_in_rad)*c_xcoord + np.cos(angle_in_rad)*c_ycoord
        
        rotx += center_x
        roty += center_y
        
        return rotx, roty
    
    
    def recalculate_xcoordinate_stepsize_to_perform_interpolation(self, rotated_xcoord, stepsize, new_stepsize):
        if stepsize == 0:
            interpolated_rotated_xcoord = np.linspace(rotated_xcoord.min(), rotated_xcoord.max(), 3)
        elif new_stepsize:
            interpolated_rotated_xcoord = np.arange(rotated_xcoord.min(), rotated_xcoord.max(), stepsize)
        else:
            interpolated_rotated_xcoord = np.linspace(rotated_xcoord.min(), rotated_xcoord.max(), len(rotated_xcoord))

        return interpolated_rotated_xcoord
    

    def perform_interpolation_of_ycoordinates(self, rotated_xcoord, rotated_ycoord, interpolated_rotated_xcoord,
    segfile):

        # spline fit only works for sorted xcoord
        if rotated_xcoord.min() != rotated_xcoord[0]:
            rotated_xcoord = np.flipud(rotated_xcoord)
            rotated_ycoord = np.flipud(rotated_ycoord)

        if 2 <= len(rotated_xcoord) <= 3 or np.std(rotated_ycoord) == 0:
            spline_coefficients = interpolate.splrep(rotated_xcoord, rotated_ycoord, k=1, s=10)
        elif len(rotated_xcoord) >= 4:
            try:
                data_count = len(rotated_ycoord)
                smoothing = max(5, data_count/10)
                spline_coefficients = interpolate.splrep(rotated_xcoord, rotated_ycoord, k=3, s=smoothing)
#                weights = 1/np.std(rotated_ycoord)*np.ones(data_count)
#                smoothing = min(5, data_count/10)
#                spline_coefficients = interpolate.splrep(rotated_xcoord, rotated_ycoord, k=3, s=smoothing, w=weights)
            except(ValueError):
                msg = 'Input coordinates could not be used for interpolating new helix path. Please check integrity ' +\
                'of all helix path coordinates in {0}. The helix path must not follow a U-turn, while '.format(segfile)
                'C- and S-paths are accepted. If in doubt simply select helix coordinates again.'
                raise ValueError(msg)
        else:
            errstring = 'Chosen helix {0} too short to segment compute reliable helixpath'.format(segfile)
            self.log.errlog(errstring)
            raise ValueError(errstring)

        interpolated_rotated_ycoord = interpolate.splev(interpolated_rotated_xcoord, spline_coefficients)

        return interpolated_rotated_ycoord
    

    def perform_interpolation_derivative_for_in_plane_angle_rotation(self, interpolated_rotated_xcoord,
    interpolated_rotated_ycoord, rotated_xcoord, rotated_ycoord, segfile):
        # compute first derivative for in-plane rotation angle determination
        if 2 <= len(interpolated_rotated_xcoord) <= 3:
            spline_coefficients_derivative = interpolate.splrep(interpolated_rotated_xcoord,
            interpolated_rotated_ycoord, k=1, s=50)
        elif len(interpolated_rotated_xcoord) >= 4:
            spline_coefficients_derivative = interpolate.splrep(interpolated_rotated_xcoord,
            interpolated_rotated_ycoord, k=3, s=10)
        else:
            errstring = 'Chosen segmentation step in helix {0} too long to compute reliable helixpath'.format(segfile)
            self.log.errlog(errstring)
            raise ValueError(errstring)

        fitted_1st_derivative = interpolate.splev(interpolated_rotated_xcoord, spline_coefficients_derivative, der=1)
        try:
            fitted_2nd_derivative = interpolate.splev(interpolated_rotated_xcoord, spline_coefficients_derivative,
            der=2)
        except:
            fitted_2nd_derivative = np.zeros(fitted_1st_derivative.size)

        # backflip coordinates to original order
        if rotated_xcoord.min() != rotated_xcoord[0]:
            interpolated_rotated_xcoord = np.flipud(interpolated_rotated_xcoord)
            interpolated_rotated_ycoord = np.flipud(interpolated_rotated_ycoord)
            fitted_1st_derivative = np.flipud(fitted_1st_derivative)
            fitted_2nd_derivative = np.flipud(fitted_2nd_derivative)

        return interpolated_rotated_xcoord, interpolated_rotated_ycoord, fitted_1st_derivative, fitted_2nd_derivative
    

    def write_boxfile(self, interpolated_xcoord=None, interpolated_ycoord=None, segsizepix=None, filename=None):
        """
        * Function to write out new boxparameters

        #. Input: interpolated x- and y-array, segment size (pixel), filename
        #. Output: written file filename
        #. Usage: write_boxfile(interpolated_xcoord, interpolated_ycoord, \
        segsizepix, filename)
        """

        interpolated_xcoord = interpolated_xcoord - segsizepix/2
        interpolated_ycoord = interpolated_ycoord - segsizepix/2

        ofile = open(filename, 'a')
        for index, xcoord in enumerate(interpolated_xcoord):
            interpolated_xcoord[index], interpolated_ycoord[index]
            fivecol = 0
            if index == 0:
                fivecol = -1
            elif index == (len(interpolated_xcoord) - 1):
                fivecol = -2
            ofile.write('{0}\t{1}\t{2}\t{3}\t{4}\n'.format(int(round(interpolated_xcoord[index])),
            int(round(interpolated_ycoord[index])), segsizepix, segsizepix, fivecol))
        ofile.close()
        
        return filename
    

    def compute_persistence_length_m_from_coordinates_A(self, coord_x_A, coord_y_A):
        """
        >>> from spring.segment2d.segment import Segment
        >>> s = Segment()
        >>> coord_x_A = np.arange(20) 
        >>> coord_y_A = coord_x_A ** 2
        >>> s.compute_persistence_length_m_from_coordinates_A(coord_x_A, coord_y_A)
        5.1463148473997033e-06
        >>> coord_y_A = coord_x_A ** 1
        >>> s.compute_persistence_length_m_from_coordinates_A(coord_x_A, coord_y_A)
        1.0
        >>> coord_y_A = 10 * np.sin(coord_x_A)
        >>> s.compute_persistence_length_m_from_coordinates_A(coord_x_A, coord_y_A)
        0.0
        """
        length = 1e-10 * np.sqrt((coord_x_A[0] - coord_x_A[-1]) ** 2 + (coord_y_A[0] - coord_y_A[-1]) ** 2)
        contour = 1e-10 * self.compute_cumulative_distances_from_start_of_helix(coord_x_A, coord_y_A)[-1]

        ratio = length / contour
        if ratio <= 1 / np.sqrt(2):
            pers_length_m = 0.0
        elif 1 / np.sqrt(2) < ratio and ratio < 1.0:
            pers_length_m = 1 / ((-np.log(2 * (length / contour) ** 2 - 1) / contour))
            pers_length_m = np.min([1.0, pers_length_m])
        elif ratio >= 1.0:
            pers_length_m = 1.0

        return pers_length_m


    def interpolate_coordinates(self, xcoord, ycoord, pixelsize, stepsize, helixwidth, segfile, new_stepsize=True):
        """
        * Function to interpolate x and y coordinates according to new stepsize

        #. Input: xarray and yarray of segment coordinates, pixelsize, stepsize \
        and helixwidth in Angstrom
        #. Output: interpolated x and y-array, computed in-plane angle
        #. Usage: interpolated_xcoord, interpolated_ycoord, ipangle = \
        interpolate_coordinates(xcoord, ycoord, pixelsize, stepsize, helixwidth, segfile)

        >>> import numpy as np
        >>> from spring.segment2d.segment import Segment
        >>> s = Segment()
        >>> i_xcoord, i_ycoord, in_plane_angle, curvature = \
            s.interpolate_coordinates(np.arange(10), np.arange(10), 1, 0.5, 3, 'test.box')
        >>> np.round(i_xcoord, 2)
        array([-0.  ,  0.35,  0.71,  1.06,  1.41,  1.77,  2.12,  2.47,  2.83,
                3.18,  3.54,  3.89,  4.24,  4.6 ,  4.95,  5.3 ,  5.66,  6.01,
                6.36,  6.72,  7.07,  7.42,  7.78,  8.13,  8.49,  8.84])

        >>> np.round(i_ycoord, 2)
        array([ 0.  ,  0.35,  0.71,  1.06,  1.41,  1.77,  2.12,  2.47,  2.83,
                3.18,  3.54,  3.89,  4.24,  4.6 ,  4.95,  5.3 ,  5.66,  6.01,
                6.36,  6.72,  7.07,  7.42,  7.78,  8.13,  8.49,  8.84])
        >>> np.round(in_plane_angle, 2)
        array([ 135.,  135.,  135.,  135.,  135.,  135.,  135.,  135.,  135.,
                135.,  135.,  135.,  135.,  135.,  135.,  135.,  135.,  135.,
                135.,  135.,  135.,  135.,  135.,  135.,  135.,  135.])
                
        >>> interpolated_x_coordinates, interpolated_y_coordinates, in_plane_rotation_angle, curvature = \
                                    Segment().interpolate_coordinates(np.arange(10), 2*np.arange(10), 1, 3, 3, 'test.box')
        >>> np.round(interpolated_x_coordinates, 1)
        array([-0. ,  1.3,  2.7,  4. ,  5.4,  6.7,  8. ])
        
        >>> np.round(interpolated_y_coordinates, 1)
        array([  0. ,   2.7,   5.4,   8. ,  10.7,  13.4,  16.1])
        
        >>> in_plane_rotation_angle
        array([ 153.43494882,  153.43494882,  153.43494882,  153.43494882,
                153.43494882,  153.43494882,  153.43494882])

        >>> # helix parallel to image rows
        ...
        >>> interpolated_x_coordinates, interpolated_y_coordinates, in_plane_rotation_angle, curvature = \
                                    Segment().interpolate_coordinates(np.arange(10), np.zeros(10), 1, 3, 3, 'test.box')
        >>> interpolated_x_coordinates
        array([ 0.,  3.,  6.])
        
        >>> np.round(interpolated_y_coordinates, 2)
        array([ 0.,  0.,  0.])
        
        >>> in_plane_rotation_angle
        array([ 90.,  90.,  90.])
        >>> # helix parallel to image columns
        ...
        >>> interpolated_x_coordinates, interpolated_y_coordinates, in_plane_rotation_angle, curvature = \
                                    Segment().interpolate_coordinates(np.zeros(10), np.arange(10), 10, 20, 30, 'test.box')
        
        >>> np.round(interpolated_y_coordinates, 0)
        array([ 0.,  2.,  4.,  6.,  8.])
        
        >>> in_plane_rotation_angle
        array([ 180.,  180.,  180.,  180.,  180.])
        
        >>> np.round(interpolated_x_coordinates, 2)
        array([-0.,  0.,  0.,  0.,  0.])

        """
        xcoord_A = xcoord * pixelsize
        ycoord_A = ycoord * pixelsize
        
        flat_angle = self.determine_angle_to_rotate_coordinates_to_minimal_slope(xcoord_A, ycoord_A, helixwidth)
        rotated_xcoord, rot_ycoord = self.rotate_coordinates_by_angle(xcoord_A, ycoord_A, flat_angle)
        
        if len(xcoord_A) > 4 and np.std(rot_ycoord) < 100:
            poly = np.polyfit(rotated_xcoord, rot_ycoord, 3)
            rotated_ycoord = np.polyval(poly, rotated_xcoord)
        else:
            rotated_ycoord = rot_ycoord

#        self.log.ilog('Coordinates are rotated by {0} degrees'.format(np.degrees(flat_angle)))

        interpolated_rotated_xcoord = self.recalculate_xcoordinate_stepsize_to_perform_interpolation(rotated_xcoord,
        stepsize, new_stepsize)
        
        interpolated_rotated_ycoord = self.perform_interpolation_of_ycoordinates(rotated_xcoord, rotated_ycoord,
        interpolated_rotated_xcoord, segfile)
        
        interpolated_rotated_xcoord, interpolated_rotated_ycoord, fitted_1st_derivative, fitted_2nd_derivative = \
        self.perform_interpolation_derivative_for_in_plane_angle_rotation(interpolated_rotated_xcoord,
        interpolated_rotated_ycoord, rotated_xcoord, rotated_ycoord, segfile)
        
        # backrotate coordinates
        interpolated_xcoord_A, interpolated_ycoord_A = self.rotate_coordinates_by_angle(interpolated_rotated_xcoord,
        interpolated_rotated_ycoord, -flat_angle)
        
        ipangle = 90 + np.rad2deg(np.arctan(fitted_1st_derivative)) + flat_angle
        pers_length = self.compute_persistence_length_m_from_coordinates_A(interpolated_xcoord_A, interpolated_ycoord_A)
        curvature = len(interpolated_rotated_xcoord) * [pers_length]

        interpolated_xcoord = interpolated_xcoord_A / pixelsize
        interpolated_ycoord = interpolated_ycoord_A / pixelsize
        
        return interpolated_xcoord, interpolated_ycoord, ipangle, curvature
    
    
class SegmentStraighten(SegmentInterpolation):
    def select_coordinates_within_segment_dimensions(self, coordinates, current_center, segment_size_pix):
        """
        >>> from spring.segment2d.segment import Segment
        >>> Segment().select_coordinates_within_segment_dimensions(zip(np.arange(20.), np.arange(20., 40.)), (10, 30), 10.0)
        (array([  6.,   7.,   8.,   9.,  10.,  11.,  12.,  13.,  14.]), array([ 26.,  27.,  28.,  29.,  30.,  31.,  32.,  33.,  34.]))
        """
        x_coordinates, y_coordinates = list(zip(*coordinates))
        x_coordinates = np.array(x_coordinates)
        y_coordinates = np.array(y_coordinates)
        
        x_center, y_center = current_center
        x_left_corner = x_center - segment_size_pix/2 
        y_left_corner = y_center - segment_size_pix/2
        x_right_corner = x_center + segment_size_pix/2
        y_right_corner = y_center + segment_size_pix/2
        
        x_indices = np.arange(x_coordinates.size)
        y_indices = np.arange(y_coordinates.size)
        within_xlimits = x_indices[(x_coordinates > x_left_corner) & (x_coordinates < x_right_corner)]
        within_ylimits = y_indices[(y_coordinates > y_left_corner) & (y_coordinates < y_right_corner)]
        
        within_x_and_y = np.intersect1d(within_xlimits, within_ylimits)
        
        filtered_x = np.array([x_coordinates[each_index] for each_index in within_x_and_y])
        filtered_y = np.array([y_coordinates[each_index] for each_index in within_x_and_y])
        
        return filtered_x, filtered_y
        
    
    def compute_bending_path_row_wise(self, length, polyfit, angle):
        """
        >>> from spring.segment2d.segment import Segment
        >>> Segment().compute_bending_path_row_wise(20.0, [1/20.0, -1.0, 15.0], 0)
        (array([ 15.  ,  14.05,  13.2 ,  12.45,  11.8 ,  11.25,  10.8 ,  10.45,
                10.2 ,  10.05,  10.  ,  10.05,  10.2 ,  10.45,  10.8 ,  11.25,
                11.8 ,  12.45,  13.2 ,  14.05]), array([  0.,   1.,   2.,   3.,   4.,   5.,   6.,   7.,   8.,   9.,  10.,
                11.,  12.,  13.,  14.,  15.,  16.,  17.,  18.,  19.]), array([ 315.        ,  318.0127875 ,  321.34019175,  325.0079798 ,
                329.03624347,  333.43494882,  338.19859051,  343.30075577,
                348.69006753,  354.28940686,  360.        ,    5.71059314,
                 11.30993247,   16.69924423,   21.80140949,   26.56505118,
                 30.96375653,   34.9920202 ,   38.65980825,   41.9872125 ]))

        >>> Segment().compute_bending_path_row_wise(20.0, [1/20.0, -1.0, 15.0], 90)
        (array([ 20.,  19.,  18.,  17.,  16.,  15.,  14.,  13.,  12.,  11.,  10.,
                 9.,   8.,   7.,   6.,   5.,   4.,   3.,   2.,   1.]), array([ 15.  ,  14.05,  13.2 ,  12.45,  11.8 ,  11.25,  10.8 ,  10.45,
                10.2 ,  10.05,  10.  ,  10.05,  10.2 ,  10.45,  10.8 ,  11.25,
                11.8 ,  12.45,  13.2 ,  14.05]), array([ 225.        ,  228.0127875 ,  231.34019175,  235.0079798 ,
                239.03624347,  243.43494882,  248.19859051,  253.30075577,
                258.69006753,  264.28940686,  270.        ,  275.71059314,
                281.30993247,  286.69924423,  291.80140949,  296.56505118,
                300.96375653,  304.9920202 ,  308.65980825,  311.9872125 ]))
        """
        straight_rows = np.arange(float(length))
        straight_cols = np.polyval(polyfit, straight_rows)
        
        spline_coefficients = interpolate.splrep(straight_rows, straight_cols, k=3, s=10)
        fitted_first_derivative = interpolate.splev(straight_rows, spline_coefficients, der=1)

        inplane_angles = (np.rad2deg(np.arctan(fitted_first_derivative)) - angle)%360
        
        cols, rows = self.rotate_coordinates_by_angle(straight_cols, straight_rows, -angle, center_x=length/2,
        center_y=length/2)
        
        return cols, rows, inplane_angles
    
    
    def shift_and_rotate_image_linear(self, image, inplane_angle, each_x, each_y):
        t_shift = Transform({'type':'SPIDER', 'psi':0.0, 'tx':each_x, 'ty':each_y, 'scale':1.0})
        t_rot = Transform({'type':'SPIDER', 'psi':inplane_angle, 'tx':0.0, 'ty':0.0, 'scale':1.0})
        composite_transform = t_rot * t_shift
        trans_rot_image = image.rot_scale_trans_background(composite_transform)
        
        return trans_rot_image
    
    
    def convert_rotate_shift_to_shift_rotate_order(self, inplane_angle, each_x, each_y):
        """
        >>> from spring.segment2d.segment import Segment
        >>> a, b, c = Segment().convert_shift_rotate_to_rotate_shift_order(50,22,2)
        >>> Segment().convert_rotate_shift_to_shift_rotate_order(a, b, c)
        (49.99999868188684, 22.0, 2.0)
        """
        psi, tx, ty = self.convert_shift_rotate_to_rotate_shift_order(-inplane_angle, each_x, each_y)
        
        return (-psi) % 360, tx, ty


    def convert_shift_rotate_to_rotate_shift_order(self, inplane_angle, each_x, each_y):
        """
        >>> from spring.segment2d.segment import Segment
        >>> Segment().convert_shift_rotate_to_rotate_shift_order(50,22,2)
        (49.99999868188683, 15.673417091369629, -15.567402839660645)
        """
        t_shift = Transform({'type':'SPIDER', 'psi':0.0, 'tx':each_x, 'ty':each_y, 'scale':1.0})
        t_rot = Transform({'type':'SPIDER', 'psi':inplane_angle, 'tx':0.0, 'ty':0.0, 'scale':1.0})
        composite_transform = t_rot * t_shift
        t = composite_transform.get_params('spider')

        return t['psi'], t['tx'], t['ty']


    def shift_and_rotate_image(self, image, inplane_angle, each_x, each_y):
        angle, tx, ty = self.convert_shift_rotate_to_rotate_shift_order(inplane_angle, each_x, each_y)
        trans_rot_image = image.rot_scale_trans2D_background(angle, tx, ty, 1.0)
        
        return trans_rot_image
    
    
    def shift_and_rotate_image_gridding(self, image, inplane_angle, each_x, each_y):
        trans_rot_image, kb = prepi(image)
        angle, tx, ty = self.convert_shift_rotate_to_rotate_shift_order(inplane_angle, each_x, each_y)
        trans_rot_image = trans_rot_image.rot_scale_conv_new_background(np.deg2rad(angle), tx, ty, kb, 1.0)
        
        return trans_rot_image
    
    
    def straighten_segment(self, helix_img, img_size, cols, rows, inplane_angles, mode='straighten'):
        if mode == 'bend':
            pd_img_size = 2*img_size
        else:
            pd_img_size = int(1.2*img_size)
        helix_pd = Util.pad(helix_img, pd_img_size, pd_img_size, 1, 0, 0, 0, '0')
            
        bent_helix = model_blank(img_size, img_size)
        for each_index, each_row in enumerate(rows):
            each_col = cols[each_index]
            each_angle = inplane_angles[each_index]
            if mode == 'bend':
                helix_wi = Util.window(helix_pd, img_size, img_size, 1, 0, int(each_row - img_size/2), 0)
                helix_rot = rot_shift2D(helix_wi, each_angle, (each_col - img_size/2), 0)
            elif mode == 'straighten':
                helix_rot = self.shift_and_rotate_image(helix_pd, -each_angle, -(each_col - img_size/2), 
                                                        -(each_row - img_size/2))
                
                helix_rot = Util.window(helix_rot, img_size, img_size, 1, 0, 0, 0)
            bent_row = helix_rot.get_row(img_size / 2)
            bent_helix.set_row(bent_row, each_index)
        
        return bent_helix
    
    
    def compute_rotated_helix_path_on_segment_locally(self, current_coordinates, cut_coordinates, current_inplane_angle,
    large_segsizepix, filtered_x, filtered_y):
        """
        >>> from spring.segment2d.segment import Segment
        >>> Segment().compute_rotated_helix_path_on_segment_locally((17.0, 87.0), (17, 87), 180.0, 10.0, np.arange(10, 20), np.arange(80, 90))
        (array([ 12.,  11.,  10.,   9.,   8.,   7.,   6.,   5.,   4.,   3.]), array([ 12.,  11.,  10.,   9.,   8.,   7.,   6.,   5.,   4.,   3.]))
        """
        center_x, center_y = current_coordinates
        cut_center_x, cut_center_y = cut_coordinates
        local_x = filtered_x - cut_center_x + large_segsizepix / 2
        local_y = filtered_y - cut_center_y + large_segsizepix / 2
        x_offset = center_x - cut_center_x
        y_offset = center_y - cut_center_y
        
        rot_local_x, rot_local_y = self.rotate_coordinates_by_angle(local_x, local_y, current_inplane_angle,
        large_segsizepix / 2 + x_offset, large_segsizepix / 2 + y_offset)
        
        return rot_local_x, rot_local_y
    


    def set_polyfit_variable(self, large_segsizepix, current_inplane_angle, second_order_fit):
        first_order_from_ip_angle = np.tan(np.deg2rad(-current_inplane_angle - 90))
        zero_order = large_segsizepix / 2 - (first_order_from_ip_angle * large_segsizepix / 2)
        polyfit = [second_order_fit, first_order_from_ip_angle, zero_order]
        return polyfit

    def fit_square_function_path_of_coordinates(self, large_segsizepix, helix_coordinates, current_coordinates,
    cut_coordinates, current_inplane_angle, second_order_fit=None):
        filtered_x, filtered_y = self.select_coordinates_within_segment_dimensions(helix_coordinates,
        current_coordinates, 1.5 * large_segsizepix)
        
        rot_local_x, rot_local_y = self.compute_rotated_helix_path_on_segment_locally(current_coordinates,
        cut_coordinates, current_inplane_angle, large_segsizepix, filtered_x, filtered_y)
        
        if second_order_fit is None:
            if rot_local_y != []:
                polyfit = np.polyfit(rot_local_y, rot_local_x, 2)
            else:
                polyfit = self.set_polyfit_variable(large_segsizepix, current_inplane_angle, second_order_fit=0)
        elif second_order_fit is not None:
            corrected_local_x = rot_local_x - (second_order_fit * rot_local_y**2)
            if rot_local_y != [] or corrected_local_x != []:
                polyfit = np.polyfit(rot_local_y, corrected_local_x, 1)
                polyfit = np.insert(polyfit, 0, second_order_fit)
            else:
                polyfit = self.set_polyfit_variable(large_segsizepix, current_inplane_angle, second_order_fit)
            
        return polyfit, filtered_x, filtered_y


    def unbend_segment_using_coordinates(self, segment, large_segsizepix, helix_coordinates, current_coordinates,
    cut_coordinates, current_inplane_angle, second_order_fit=None):
        polyfit, filtered_x, filtered_y = self.fit_square_function_path_of_coordinates(large_segsizepix,
        helix_coordinates, current_coordinates, cut_coordinates, current_inplane_angle, second_order_fit)
        
        cols, rows, inplane_angles = self.compute_bending_path_row_wise(large_segsizepix, polyfit,
        current_inplane_angle)
        
        straightened_segment = self.straighten_segment(segment, large_segsizepix, cols, rows, inplane_angles)
        central_inplane_angle = inplane_angles[len(inplane_angles)/2]
        
        return straightened_segment, polyfit, central_inplane_angle
        



#!/usr/bin/env python
"""
Test module to check segmultirefine3d
"""
from glob import glob
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment3d.segmultirefine3d import SegMultiRefine3dPar, SegMultiRefine3d
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment3d.test_segmentrefine3d import TestSegmentRefine3dClean, TestSegmentRefine3dSetup
from utilities import model_gauss_noise
import os

        
class TestSegMultiRefine3dSetup(TestSegmentRefine3dSetup):
        
    def setup_of_common_sr3dg_non_mpi_parameters(self):
        self.feature_set = SegMultiRefine3dPar()
        self.input_stack = 'test_stack.hdf'
        self.setup_main_segmentrefine3d_parameters()
        
        self.feature_set.parameters.__delitem__('Helical symmetry in Angstrom or degrees')
        self.feature_set.parameters.__delitem__('Reference volume')
        self.feature_set.parameters.__delitem__('Rotational symmetry')
        self.feature_set.parameters.__delitem__('Helix polarity')
        self.feature_set.parameters.__delitem__('Filter layer-lines option')

        self.feature_set.parameters['Helical symmetries in Angstrom or degrees']='(10, 50); (20, 80); (30, 30)'
        self.feature_set.parameters['Reference volumes']='models???.hdf'
        self.feature_set.parameters['Rotational symmetries']='1, 1, 2'
        self.feature_set.parameters['Helix polarities']='polar, apolar, polar'
        
        
class TestSegMultiRefine3d(TestSegMultiRefine3dSetup, SegMultiRefine3d):
    def setup_srg3d(self):
        self.setup_of_common_sr3dg_non_mpi_parameters()
        
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(2, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)
        
        self.prepare_apolar_helix = False
        self.generate_input_stack_from_helix()
        
        super(TestSegMultiRefine3dSetup, self).__init__(self.feature_set)
        
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.setup_srg3d()
        
    def teardown(self):
        self.cleanup_segmentrefine3d()

        self.testingdir.remove()
            
        
class TestSegMultiRefine3dMain(TestSegMultiRefine3d):
    def do_test_case_smr3d1(self):
        """
        * Standard single-CPU segmultirefine3d test
        """
        self.perform_iterative_projection_matching_and_3d_reconstruction()

        models = glob('{prefix}*{iter:03}{ext}'.format(prefix=os.path.splitext(self.test_reference_volume)[0],
        iter=self.iteration_count, ext=os.path.splitext(self.test_reference_volume)[1]))

        TestSegmentRefine3dClean().prepare_final_prj_for_documentation(models, self.test_reference_volume)


class TestSegMultiRefine3dMore(TestSegMultiRefine3d):
    def do_test_case_smr3d2(self):
        """
        * Standard single-CPU segmultirefine3d test with single reference
        """
        self.feature_set.parameters['Reference structure option']=True
        self.feature_set.parameters['Reference volumes']='models000.hdf'
        vol = model_gauss_noise(1, 100, 100, 100)
        vol.write_image('models000.hdf')

        super(TestSegMultiRefine3dSetup, self).__init__(self.feature_set)

        self.perform_iterative_projection_matching_and_3d_reconstruction()
        os.remove('models000.hdf')


class TestSegMultiRefine3dMpi(TestSegMultiRefine3d):
    def do_end_to_end_test_smr3d_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
         
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

class TestSegMultiRefine3dEndToEnd(TestSegMultiRefine3d):
    def do_end_to_end_test_smr3d_inputfile(self):

        references = ['models001.hdf', 'models002.hdf', 'models003.hdf']
        for each_model in references:
            vol = model_gauss_noise(1, 100, 100, 100)
            vol.write_image(each_model)

        self.feature_set.parameters['Reference structure option']=True
        self.feature_set.parameters['Reference volumes']=', '.join([os.path.abspath(each) for each in references])
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 
        [os.remove(each_model) for each_model in references]


    def do_end_to_end_test_smr3d_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
        
def main():
    tsr3d = TestSegMultiRefine3dMain()
    tsr3d.setup()
    tsr3d.do_test_case_smr1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details

from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment3d.segclassmodel import SegClassModelPar, SegClassModel
import json
import os


class SegClassModelMpi(SegClassModel):
    def prepare_merged_stack_of_projections_mpi(self):
        sr3d = self.prepare_sr3d_object_with_filter_settings()

        projection_size, projection_info = self.prepare_prj_through_series_of_models()

        for each_model_id, each_reference in enumerate(self.references):
            if self.rank == 0:
                reference_info, pixelinfo = self.prepare_volume_for_projection(projection_size, sr3d, each_model_id, each_reference)
                
                reference_info = list(reference_info)
                pixelinfo = list(pixelinfo)
            else:
                reference_info = None
                pixelinfo = None
            
            reference_info = self.comm.bcast(reference_info, root=0)
            pixelinfo = self.comm.bcast(pixelinfo, root=0)
            
            pixelinfo_nt = sr3d.make_pixel_info_named_tuple()
            reference_info_nt = sr3d.make_reference_info_named_tuple()
            
            pixelinfo = pixelinfo_nt._make(pixelinfo)
            reference_info = reference_info_nt._make(reference_info)

            projection_stack, projection_parameters, fine_projection_stack, fine_projection_parameters = \
            sr3d.project_through_reference_volume_in_helical_perspectives('medium', reference_info.model_id,
            reference_info.ref_file, pixelinfo, reference_info.helical_symmetry, reference_info.rotational_symmetry)

            if self.rank == 0:
                projection_info = self.summarize_prj_info(projection_info, each_model_id, projection_stack,
                projection_parameters)
            
            self.comm.barrier()
        
        if self.rank == 0:
            merged_prj_stack, projection_info = self.merge_prj_stacks_and_collect_prj_info(projection_info)

            dfile = open('prj_info.dat', 'w')
            json.dump(projection_info, dfile, indent=4, sort_keys=False)
            dfile.close()
        
        else:
            merged_prj_stack = None
            projection_info = None

        self.comm.barrier()

        return  merged_prj_stack, projection_info


    def match_reprojections_to_classes(self):
        self.comm, self.rank, self.size, self.log, self.tempdir = OpenMpi().setup_mpi_and_simultaneous_logging(self.log,
        self.feature_set.logfile, self.temppath)
        
        merged_prj_stack, prj_info = self.prepare_merged_stack_of_projections_mpi()
        
        os.rmdir(self.tempdir)
        if self.rank == 0:
            self.log.endlog(self.feature_set)
        

def main():
    parset = SegClassModelPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    class_average = SegClassModelMpi(reduced_parset)
    class_average.match_reprojections_to_classes()


if __name__ == '__main__':
    main()



# Author: Carsten Sachse 29-Aug-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
"""
Program to interactively explore refinement cycle statistics
"""
from spring.csinfrastr.csdatabase import SpringDataBase, refine_base, RefinementCycleTable
from spring.csinfrastr.csfeatures import Features
from spring.csinfrastr.cslogger import Logger
from spring.csinfrastr.csproductivity import DiagnosticPlot
from spring.csinfrastr.csreadinput import OptHandler
from spring.segment2d.segmentplot import SegmentPlot
from spring.segment3d.seggridexplore import SegGridExploreSupport
from sqlalchemy.sql.expression import asc
import numpy as np
import os
import shutil


class SegRefine3dCycleExplorePar(object):
    """
    Class to initiate default dictionary with input parameters including help and range values and status dictionary
    """
    def __init__(self):
        # package/program identity
        self.package = 'emspring'       
        self.progname = 'segrefine3dcyclexplore'
        self.proginfo = __doc__
        self.code_files = [self.progname]

        self.seggridexplore_features = Features()
        self.feature_set = self.seggridexplore_features.setup(self)
        
        self.define_parameters_and_their_properties()
        self.define_program_states()


    def define_parameters_and_their_properties(self):
        self.feature_set = self.seggridexplore_features.set_inp_refinement_path(self.feature_set)
        self.feature_set = self.seggridexplore_features.set_interactive_vs_batch_mode(self.feature_set)
        
        self.feature_set = self.seggridexplore_features.set_diagnostic_prefix(self.feature_set, 'intermediate', 
        'Batch mode')
        
        self.feature_set = self.set_criterion_to_be_plotted(self.feature_set)
        
    def define_program_states(self):
        self.feature_set.program_states['extract_desired_criteria']='Extract desired refinement criteria'
        self.program_states['launch_segrefine3dcyclexplore_gui']='Explore different refinement criteria interactively'

    

    def set_criterion_to_be_plotted(self, feature_set):
        inp9 = 'Criterion'
        feature_set.parameters[inp9]=str('mean_helical_ccc')
        
        criteria = SegRefine3dCycleExploreSupport().get_segrefin3dcyclexpore_criteria()
        
        feature_set.properties[inp9] = feature_set.choice_properties(2, criteria, 'QComboBox')
        
        feature_set.hints[inp9]=SpringDataBase().get_hints_from_refinement_cycle(criteria)
        feature_set.relatives[inp9]=('Batch mode')
        feature_set.level[inp9]='intermediate'
        
        return feature_set
    
    
class SegRefine3dCycleExploreSupport(object):
    def get_segrefin3dcyclexpore_criteria(self):
        criteria = SegGridExploreSupport().get_common_criterias() + \
        SegRefine3dCycleExploreSupport().get_additional_criteria()
        
        return criteria


    def get_additional_criteria(self):
        return ['excluded_out_of_plane_tilt_count', 'excluded_inplane_count', 'total_excluded_count']
    
    
class SegRefine3dCycleExplorePreparation(object):
    def __init__(self, parset = None):
        self.log = Logger()
        if parset is not None:
            self.feature_set = parset
            p = self.feature_set.parameters
            
            self.refinement_db = p['refinement.db file']
            self.infile = self.refinement_db
            self.batch_mode = p['Batch mode']
            self.outfile_prefix = p['Diagnostic plot prefix']
            
            self.criterion = p['Criterion']
            
            
    def get_refinement_info_from_database(self, refinement_db):
        ref_session = SpringDataBase().setup_sqlite_db(refine_base, refinement_db)
        ref_cycles = ref_session.query(RefinementCycleTable).order_by(asc(RefinementCycleTable.id)).all()
        
        ref_data = []
        ylabels = []
        filtered_quant = []
        
        criteria = SegRefine3dCycleExploreSupport().get_segrefin3dcyclexpore_criteria()
        
        for each_attr in criteria:
            ref_data, ylabels, filtered_quant = SegGridExploreSupport().get_all_attr_from_search_result(ref_cycles,
            each_attr, filtered_quant, ref_data, ylabels, RefinementCycleTable)
            
        ref_cycle_ids = [each_ref_cycle.id for each_ref_cycle in ref_cycles]
        
        xlabel = 'Iteration cycle'
        
        ref_data = [(ylabels[each_id], np.array([ref_cycle_ids, each_ref_data])) for (each_id, each_ref_data) in\
        enumerate(ref_data)]
        
        labels = [[xlabel, each_y_label] for each_y_label in ylabels]
        
        hints = SpringDataBase().get_hints_from_refinement_cycle(filtered_quant)
        
        return ref_data, labels, criteria, hints
    
    
class SegRefine3dCycleExplore(SegRefine3dCycleExplorePreparation):
    
    def plot_data_on_figure(self, xy_quantities, labels, criterion_id):
        self.log.fcttolog()
        
        figures = []
        segmentplot_plot = DiagnosticPlot()
        self.log.plog(20)
        for quant_index, (each_set_id_label, each_segid_xy) in enumerate(xy_quantities):
            fig = segmentplot_plot.create_next_figure()
            
            filename = '{0}_{1}{2}'.format(os.path.splitext(self.outfile_prefix)[0],
            '_'.join(each_set_id_label.lower().split()), os.path.splitext(self.outfile_prefix)[-1])
            
            if self.batch_mode:
                fig = segmentplot_plot.add_header_and_footer(self.feature_set, self.infile, filename)
        
            ax1 = fig.add_subplot(111)
            
            ax1.set_title('{0}'.format(each_set_id_label.title()))
            x_label, y_label = labels[quant_index]
            x_data, y_data = each_segid_xy
            ax1.plot(x_data, y_data, 'x')
            ax1.plot(x_data, y_data, ':')
            [each_tick.set_fontsize(8) for each_tick in ax1.get_xticklabels() + ax1.get_yticklabels()]
            
            ax1.set_xlabel(x_label, fontsize=8)
            ax1.set_ylabel(y_label, fontsize=8)
                    
            if self.batch_mode and quant_index in criterion_id:
                fig.savefig(filename)
            elif not self.batch_mode:
                figures.append(fig)
            self.log.plog(70 * (quant_index + 1) / len(xy_quantities) + 20)
            self.log.ilog('{0}{1} plot was generated.'.format(each_set_id_label[0].capitalize(), each_set_id_label[1:]))
        
        return figures
    
    
        
    def plot_refinement_criteria_per_cycle(self):
        work_refinement_db = os.path.basename(self.refinement_db)
        shutil.copy(self.refinement_db, work_refinement_db)
            
        xy_quantities, labels, criteria, hints = self.get_refinement_info_from_database(work_refinement_db)
        
        criterion_id = [crit_id for (crit_id, each_criterion) in enumerate(criteria) if each_criterion == self.criterion]
        
        self.log.plog(10)
        figures = self.plot_data_on_figure(xy_quantities, labels, criterion_id)
        if not self.batch_mode:
            SegmentPlot().launch_interactive_plot_gui(self.feature_set, figures, xy_quantities, labels, hints)
        
        self.log.endlog(self.feature_set)
        
        
def main():
    # Option handling
    parset = SegRefine3dCycleExplorePar()
    mergeparset = OptHandler(parset)

    ######## Program
    sseg = SegRefine3dCycleExplore(mergeparset)
    sseg.plot_refinement_criteria_per_cycle()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check segrefine3dplot
"""
from EMAN2 import EMData
from filter import filt_table
import os
from projection import prj
from spring.csinfrastr.csdatabase import SpringDataBase, refine_base, RefinementCycleTable
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.segment2d.segmentalign2d import SegmentAlign2d
from spring.segment3d.refine.sr3d_main import SegmentRefine3d
from spring.segment3d.segrefine3dinspect import SegRefine3dInspect, SegRefine3dInspectPar
from spring.segment3d.segrefine3dinspect_gui import SegRefine3dInspectCommonOperations
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.segment3d.test_segclassreconstruct import TestSegClassReconstructPreparation
from utilities import model_gauss_noise


class TestSegRefine3dInspectPrepare(object):

    def simulate_fsc_line(self):
        ff = SegmentAlign2d().prepare_filter_function(False, 0.02, True, 0.08, self.pixelsize, self.segment_size)
        return ff

    def setup_segrefine3dinspect(self):
        if self.high_res_test:
            scale_factor = 2.0
            self.pixelsize /= scale_factor
            self.segment_size *= int(scale_factor)

        helix_volume = TestSegClassReconstructPreparation().generate_helical_volume(self.size_of_subunit_in_angstrom,
        self.pixelsize, self.segment_size, self.helix_radius_in_angstrom, self.helical_rise, self.helical_rotation)
        
        res_cutoff= 2 * self.pixelsize

        filter_coefficients = SegmentAlign2d().prepare_bfactor_coefficients(self.test_bfactor, self.pixelsize,
        self.segment_size, res_cutoff)
        
        helix_volume = filt_table(helix_volume, filter_coefficients.tolist())

        noise = model_gauss_noise(0.001, self.segment_size, self.segment_size, self.segment_size)
        helix_volume += noise
        helix_volume.write_image(self.helix_volume)

        ff = self.simulate_fsc_line()
        f_nt = SegmentRefine3d().make_fsc_line_named_tuple()

        self.test_fsc_file = SegmentRefine3d().write_out_fsc_line(f_nt(ff, ff, len(ff) * [None], len(ff) * [None]), 
        self.pixelsize, 'test_fsc.dat', 3)


class TestSegRefine3dInspect(TestSegRefine3dInspectPrepare, SegRefine3dInspect):
    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.helix_volume = 'helix_volume.hdf'
        self.processed_volume = 'test_processed_vol.hdf'
        self.size_of_subunit_in_angstrom = 50
        self.pixelsize = 5.0
        self.segment_size = 50
        self.helix_radius_in_angstrom = 70
        self.helical_rise = 10.0
        self.helical_rotation = 50.0
        self.test_bfactor = 300.0
        self.high_res_test = True
        self.setup_segrefine3dinspect()
        
        self.feature_set = SegRefine3dInspectPar()
        self.feature_set.parameters['Volume reconstruction']=self.helix_volume
        self.feature_set.parameters['Batch mode']=True#False#
        self.feature_set.parameters['Output volume name']= self.processed_volume
        
        self.feature_set.parameters['B-factor']=True
        self.feature_set.parameters['B-factor and resolution cutoff']=((-self.test_bfactor, 2.5 * self.pixelsize))
        self.feature_set.parameters['Signal-to-noise weighting']=True
        self.feature_set.parameters['FSC curve']=self.test_fsc_file
        self.feature_set.parameters['Pixel size in Angstrom']=self.pixelsize
        self.feature_set.parameters['Estimated helix inner and outer diameter in Angstrom']=((50, 200))
        self.feature_set.parameters['Real-space mask']=True
        self.feature_set.parameters['Mask type']='cylinder'
        self.feature_set.parameters['Layer-line Fourier filter']=True
        self.feature_set.parameters['Helical rise/rotation or pitch/number of units per turn choice']='rise/rotation'
        
        self.feature_set.parameters['Helical symmetry in Angstrom or degrees']=((self.helical_rise,
        self.helical_rotation))
        
        self.feature_set.parameters['Rotational symmetry']=1
        self.feature_set.parameters['Helix polarity']='apolar'
        
        self.feature_set.parameters['Long helix']=False
        self.feature_set.parameters['Helix length in Angstrom']=500
        self.feature_set.parameters['Swap handedness']=True
        
        super(TestSegRefine3dInspect, self).__init__(self.feature_set)
        
        
    def teardown(self):
        os.remove(self.helix_volume)
        os.remove(self.test_fsc_file)
        if self.batch_mode:
            os.remove(self.processed_volume)
        
        self.testingdir.remove()
        

class TestSegRefine3dInspectMain(TestSegRefine3dInspect):
    def do_test_case_sr3di1(self):
        self.launch_inspection_of_reconstruction()
        
        projection_parameters = [[0., 90., 270., 0., 0.]]
        if self.batch_mode:
            vol = EMData()
            vol.read_image(self.helix_volume)
            prj(vol, projection_parameters, self.helix_volume)
            vol.read_image(self.processed_volume)
            prj(vol, projection_parameters, self.processed_volume)


class TestSegRefine3dInspectMore(TestSegRefine3dInspect):

    def setup_fsc_in_refinement_db(self):
        ff = self.simulate_fsc_line()
        fsc_file = os.path.splitext(self.test_fsc_file)[0] + os.extsep + 'db'
        self.feature_set.parameters['FSC curve'] = fsc_file
        ref_session = SpringDataBase().setup_sqlite_db(refine_base, fsc_file)
        ref_cycle = RefinementCycleTable()
        ref_cycle.fsc = list(ff)
        ref_session.add(ref_cycle)
        ref_session.commit()
        
        return fsc_file


    def do_test_case_sr3di2(self):
        """
        * Segrefine3dinspect: Test structure mask and long helix option including db fsc read
        """
        fsc_file = self.setup_fsc_in_refinement_db()

        self.feature_set.parameters['Mask type']='structure'
        self.feature_set.parameters['Long helix']=True
        super(TestSegRefine3dInspect, self).__init__(self.feature_set)

        self.launch_inspection_of_reconstruction()
        
        os.remove(fsc_file)
        
        
    def do_test_case_sr3di3(self):
        """
        * Segrefine3dinspect: No real-space mask chosen 
        """
        self.feature_set.parameters['Real-space mask']=False
        self.feature_set.parameters['Long helix']=True
        super(TestSegRefine3dInspect, self).__init__(self.feature_set)
         
        self.launch_inspection_of_reconstruction()
         
    def do_test_case_sr3di4(self):
        """
        * Segrefine3dinspect: Swap handedness without Layer-line Fourier filter
        """
        self.feature_set.parameters['Layer-line Fourier filter']=False
        self.feature_set.parameters['Swap handedness']=True
         
        super(TestSegRefine3dInspect, self).__init__(self.feature_set)
     
        self.launch_inspection_of_reconstruction()

        
    def do_test_case_sr3di5(self):
        """
        * Segrefine3dinspect: Work with higher resolution volumes to fit B-factor
        """
        self.high_res_test = True
        super(TestSegRefine3dInspect, self).__init__(self.feature_set)
     
        self.launch_inspection_of_reconstruction()


    def do_test_case_sr3di6(self):
        """
        * Segrefine3dinspect: Work with input of long volumes and pad to square 
        """

        super(TestSegRefine3dInspect, self).__init__(self.feature_set)

        vol = EMData()
        vol.read_image(self.helix_volume)
        vol = SegRefine3dInspectCommonOperations().prepare_long_helix(vol, self.helix_length, self.helixwidth,
        self.pixelsize, self.helical_symmetry, self.rotational_symmetry, self.polar_helix)
        
        vol.write_image(self.helix_volume)
        self.launch_inspection_of_reconstruction()
     

class TestSegRefine3dInspectEndToEnd(TestSegRefine3dInspect):
    def do_end_to_end_test_sp_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)
 

    def do_end_to_end_test_sp_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)
        
    
def main():
    tsr3dp = TestSegRefine3dInspectMain()
    tsr3dp.setup()
    tsr3dp.do_test_case_sr3di1()
        
if __name__ == '__main__':
    main()



# Author: Carsten Sachse 18-Aug-2011
# Copyright: EMBL (2010 - 2018)
# License: see license.txt for details
from spring.csinfrastr.csproductivity import OpenMpi
from spring.segment3d.refine.sr3d_mpi import SegmentRefine3dMpi
from spring.segment3d.segmultirefine3d import SegMultiRefine3dPar


class SegMultiRefine3dMpi(SegmentRefine3dMpi):
    pass

def main():
    parset = SegMultiRefine3dPar()
    reduced_parset = OpenMpi().start_main_mpi(parset)
    
    ####### Program
    segment_stack = SegMultiRefine3dMpi(reduced_parset)
    segment_stack.perform_iterative_projection_matching_and_3d_reconstruction()


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
Test module to check micexam
"""
from filter import filt_ctf
from multiprocessing import cpu_count
from spring.csinfrastr.csproductivity import TestingDirectory
from spring.micprgs.micexam import MicrographExam, MicrographExamPar
from spring.tests.csinfrastr.test_csreadinput import EndToEndTest
from spring.tests.micprgs.test_scansplit import ScanPreparation
from utilities import generate_ctf
import os


class TestMicrographExam(MicrographExam):
    """
    Test class that tests work of micexam
    """
    def convolute_micrograph_with_CTF(self, micrograph):
        contrast_transfer_function = generate_ctf([3.0, 2.0, 200, self.pixelsize, 0, 10])
        conv_micrograph = filt_ctf(micrograph, contrast_transfer_function)
        
        return conv_micrograph

    def setup(self):
        self.testingdir = TestingDirectory(os.path.splitext(os.path.basename(__file__))[0])
        self.testingdir.create()

        self.feature_set = MicrographExamPar()
        self.pixelsize = 3.5
        self.reference_micrograph = ScanPreparation().prepare_noise_micrograph(scan_stepsize=150.0)
        self.reference_micrograph = self.convolute_micrograph_with_CTF(self.reference_micrograph)
        self.reference_micrograph.write_image('test_noise.hdf')

        self.feature_set.parameters['Micrographs']='test_noise.hdf'
        self.feature_set.parameters['Diagnostic plot pattern']='test_diagnostic.png'
        self.feature_set.parameters['Tile size power spectrum in Angstrom']=420
        self.feature_set.parameters['Tile overlap in percent']=50
        self.feature_set.parameters['Complete tile array option']=False
        self.feature_set.parameters['Pixel size in Angstrom']=float(self.pixelsize)
        self.feature_set.parameters['Binning option']=True
        self.feature_set.parameters['Binning factor']=2
        
        self.feature_set.parameters['MPI option']=False
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        self.feature_set.parameters['Temporary directory']=os.path.abspath(os.curdir)

        super(TestMicrographExam, self).__init__(self.feature_set)


    def teardown(self):
        pass
        os.remove('test_noise.hdf')
        os.remove('test_diagnostic.png')

        self.testingdir.remove()


class TestMicrographExamMain(TestMicrographExam):
    def do_test_case_se1(self):
        self.exam_scans()


class TestMicrographExamEndToEnd(TestMicrographExam):
    def do_end_to_end_test_se_inputfile(self):
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

    def do_end_to_end_test_se_prompt(self):
        EndToEndTest().do_end_to_end_prompt(self.feature_set)

class TestMicrographExamMpi(TestMicrographExam):
    def do_end_to_end_test_sa_inputfile_with_MPI(self):
        self.feature_set.parameters['MPI option'] = True
        self.feature_set.parameters['Number of CPUs']=min(8, cpu_count())
        
        EndToEndTest().do_end_to_end_inputfile(self.feature_set)

def main():
    tse = TestMicrographExamMain()
    tse.setup()
    tse.do_test_case_se1()
        
if __name__ == '__main__':
    main()
